/*
** ADM-XRC Series Linux Driver
**
** (C) Copyright 2001-2004 Alpha Data
**
** linux_params.c - Declare the module parameters.
**
** All of the recognised module parameters must be declared here, so that they
** can be picked up as driver parameters by the generic section of the driver.
**
** ----------------------------------------------------------------------------
*/

#include "platform.h"
#include "device.h"

static long PCIReadMultiple = 1;
static long PCIWriteAndInvalidate = 1;
static long PCI9656ErrataFix = 1;
static long PCI9656DmaDisable = 0;
static long PCI9656RegsAsSpace = 0;
static long MaxDmaDesc = ADMXRC_MAX_DMADESC;
static long MaxPlxDesc = ADMXRC_MAX_PLXDESC_PER_CHANNEL;
static long CommonBufferSize = 0;
static long ICD2061PMax = 131;
static long IRQVector = 0;
static long ADMXRCMaxLCLK = 40001000;
static long ADMXRCMinLCLK = 399000;
static long ADMXRCPMaxLCLK = 40001000;
static long ADMXRCPMinLCLK = 399000;
static long ADMXRC2LMaxLCLK = 40001000;
static long ADMXRC2LMinLCLK = 399000;
static long ADMXRC2MaxLCLK = 66001000;
static long ADMXRC2MinLCLK = 399000;
static long ADMXPLMaxLCLK = 80001000;
static long ADMXPLMinLCLK = 5999000;
static long ADMXPLUnlockFlash = 0;
static long ADPWRC2MaxLCLK = 66001000;
static long ADPWRC2MinLCLK = 399000;
static long ADPDRC2MaxLCLK = 66001000;
static long ADPDRC2MinLCLK = 399000;
static long ADMXPMaxLCLK = 80001000;
static long ADMXPMinLCLK = 5999000;
static long ADMXPUnlockFlash = 0;
static long ADPXPIMaxLCLK = 80001000;
static long ADPXPIMinLCLK = 5999000;
static long ADPXPIUnlockFlash = 0;
static long ADMXRC4LSMaxLCLK = 66001000;
static long ADMXRC4LSMinLCLK = 399000;
static long ADMXRC4LSSmap32Bit = 0;
static long ADMXRC4LXMaxLCLK = 66001000;
static long ADMXRC4LXMinLCLK = 399000;
static long ADMXRC4LXSmap32Bit = 0;
static long ADMXRC4SXMaxLCLK = 66001000;
static long ADMXRC4SXMinLCLK = 399000;
static long ADMXRC4SXSmap32Bit = 0;
static long ADMXRC4FXMaxLCLK = 80001000;
static long ADMXRC4FXMinLCLK = 31999000;
static long ADMXRC4FXUnlockFlash = 0;
static long ADMXRC4FXSmap32Bit = 0;
static long ADPEXRC4FXMaxLCLK = 80001000;
static long ADPEXRC4FXMinLCLK = 31999000;
static long ADPEXRC4FXUnlockFlash = 0;
static long ADPEXRC4FXSmap32Bit = 0;
static long ADMXRC5LXMaxLCLK = 80001000;
static long ADMXRC5LXMinLCLK = 5999000;
static long ADMXRC5LXUnlockFlash = 0;
static long ADMXRC5LXSmap32Bit = 0;
static long ADMXRC5T1MaxLCLK = 80001000;
static long ADMXRC5T1MinLCLK = 31999000;
static long ADMXRC5T1UnlockFlash = 0;
static long ADMXRC5T1Smap32Bit = 0;
static long ADMXRC5T2MaxLCLK = 80001000;
static long ADMXRC5T2MinLCLK = 31999000;
static long ADMXRC5T2UnlockFlash = 0;
static long ADMXRC5T2Smap32Bit = 0;
static long ADCPXRC4LXMaxLCLK = 80001000;
static long ADCPXRC4LXMinLCLK = 31999000;
static long ADCPXRC4LXUnlockFlash = 0;
static long ADCPXRC4LXSmap32Bit = 0;
static long ADMXRC5LXAMaxLCLK = 80001000;
static long ADMXRC5LXAMinLCLK = 31999000;
static long ADMXRC5LXAUnlockFlash = 0;
static long ADMXRC5LXASmap32Bit = 0;
static long ADMAMC5A2MaxLCLK = 80001000;
static long ADMAMC5A2MinLCLK = 31999000;
static long ADMAMC5A2UnlockFlash = 0;
static long ADMAMC5A2Smap32Bit = 0;
static long ADMXRC5TZMaxLCLK = 80001000;
static long ADMXRC5TZMinLCLK = 31999000;
static long ADMXRC5TZUnlockFlash = 0;
static long ADMXRC5TZSmap32Bit = 0;
static long ADMXRC5TDA1MaxLCLK = 80001000;
static long ADMXRC5TDA1MinLCLK = 31999000;
static long ADMXRC5TDA1UnlockFlash = 0;
static long ADMXRC5TDA1Smap32Bit = 0;

static struct {
  char*           mnemonic;
  DriverParamType type;
  void*           value;
} module_params[] = {
  { "PCIReadMultiple", DriverParamType_boolean, &PCIReadMultiple },
  { "PCIWriteAndInvalidate", DriverParamType_boolean, &PCIWriteAndInvalidate },
  { "PCI9656DmaDisable", DriverParamType_boolean, &PCI9656DmaDisable },
  { "PCI9656ErrataFix", DriverParamType_boolean, &PCI9656ErrataFix },
  { "PCI9656RegsAsSpace", DriverParamType_boolean, &PCI9656RegsAsSpace },
  { "MaxDmaDesc", DriverParamType_uint32, &MaxDmaDesc },
  { "MaxPlxDesc", DriverParamType_uint32, &MaxPlxDesc },
  { "CommonBufferSize", DriverParamType_uint32, &CommonBufferSize },
  { "ICD2061PMax", DriverParamType_uint32, &ICD2061PMax },
  { "IRQVector", DriverParamType_uint32, &IRQVector },
  { "ADMXRCMaxLCLK", DriverParamType_uint32, &ADMXRCMaxLCLK },
  { "ADMXRCMinLCLK", DriverParamType_uint32, &ADMXRCMinLCLK },
  { "ADMXRCPMaxLCLK", DriverParamType_uint32, &ADMXRCPMaxLCLK },
  { "ADMXRCPMinLCLK", DriverParamType_uint32, &ADMXRCPMinLCLK },
  { "ADMXRC2LMaxLCLK", DriverParamType_uint32, &ADMXRC2LMaxLCLK },
  { "ADMXRC2LMinLCLK", DriverParamType_uint32, &ADMXRC2LMinLCLK },
  { "ADMXRC2MaxLCLK", DriverParamType_uint32, &ADMXRC2MaxLCLK },
  { "ADMXRC2MinLCLK", DriverParamType_uint32, &ADMXRC2MinLCLK },
  { "ADMXPLMaxLCLK", DriverParamType_uint32, &ADMXPLMaxLCLK },
  { "ADMXPLMinLCLK", DriverParamType_uint32, &ADMXPLMinLCLK },
  { "ADMXPLUnlockFlash", DriverParamType_boolean, &ADMXPLUnlockFlash },
  { "ADPWRC2MaxLCLK", DriverParamType_uint32, &ADPWRC2MaxLCLK },
  { "ADPWRC2MinLCLK", DriverParamType_uint32, &ADPWRC2MinLCLK },
  { "ADPDRC2MaxLCLK", DriverParamType_uint32, &ADPDRC2MaxLCLK },
  { "ADPDRC2MinLCLK", DriverParamType_uint32, &ADPDRC2MinLCLK },
  { "ADMXPMaxLCLK", DriverParamType_uint32, &ADMXPMaxLCLK },
  { "ADMXPMinLCLK", DriverParamType_uint32, &ADMXPMinLCLK },
  { "ADMXPUnlockFlash", DriverParamType_boolean, &ADMXPUnlockFlash },
  { "ADPXPIMaxLCLK", DriverParamType_uint32, &ADPXPIMaxLCLK },
  { "ADPXPIMinLCLK", DriverParamType_uint32, &ADPXPIMinLCLK },
  { "ADPXPIUnlockFlash", DriverParamType_boolean, &ADPXPIUnlockFlash },
  { "ADMXRC4LSMaxLCLK", DriverParamType_uint32, &ADMXRC4LSMaxLCLK },
  { "ADMXRC4LSMinLCLK", DriverParamType_uint32, &ADMXRC4LSMinLCLK },
  { "ADMXRC4LSSmap32Bit", DriverParamType_boolean, &ADMXRC4LSSmap32Bit },
  { "ADMXRC4LXMaxLCLK", DriverParamType_uint32, &ADMXRC4LXMaxLCLK },
  { "ADMXRC4LXMinLCLK", DriverParamType_uint32, &ADMXRC4LXMinLCLK },
  { "ADMXRC4LXSmap32Bit", DriverParamType_boolean, &ADMXRC4LXSmap32Bit },
  { "ADMXRC4SXMaxLCLK", DriverParamType_uint32, &ADMXRC4SXMaxLCLK },
  { "ADMXRC4SXMinLCLK", DriverParamType_uint32, &ADMXRC4SXMinLCLK },
  { "ADMXRC4SXSmap32Bit", DriverParamType_boolean, &ADMXRC4SXSmap32Bit },
  { "ADMXRC4FXMaxLCLK", DriverParamType_uint32, &ADMXRC4FXMaxLCLK },
  { "ADMXRC4FXMinLCLK", DriverParamType_uint32, &ADMXRC4FXMinLCLK },
  { "ADMXRC4FXUnlockFlash", DriverParamType_boolean, &ADMXRC4FXUnlockFlash },
  { "ADMXRC4FXSmap32Bit", DriverParamType_boolean, &ADMXRC4FXSmap32Bit },
  { "ADPEXRC4FXMaxLCLK", DriverParamType_uint32, &ADPEXRC4FXMaxLCLK },
  { "ADPEXRC4FXMinLCLK", DriverParamType_uint32, &ADPEXRC4FXMinLCLK },
  { "ADPEXRC4FXUnlockFlash", DriverParamType_boolean, &ADPEXRC4FXUnlockFlash },
  { "ADPEXRC4FXSmap32Bit", DriverParamType_boolean, &ADPEXRC4FXSmap32Bit },
  { "ADMXRC5LXMaxLCLK", DriverParamType_uint32, &ADMXRC5LXMaxLCLK },
  { "ADMXRC5LXMinLCLK", DriverParamType_uint32, &ADMXRC5LXMinLCLK },
  { "ADMXRC5LXUnlockFlash", DriverParamType_boolean, &ADMXRC5LXUnlockFlash },
  { "ADMXRC5LXSmap32Bit", DriverParamType_boolean, &ADMXRC5LXSmap32Bit },
  { "ADMXRC5T1MaxLCLK", DriverParamType_uint32, &ADMXRC5T1MaxLCLK },
  { "ADMXRC5T1MinLCLK", DriverParamType_uint32, &ADMXRC5T1MinLCLK },
  { "ADMXRC5T1UnlockFlash", DriverParamType_boolean, &ADMXRC5T1UnlockFlash },
  { "ADMXRC5T1Smap32Bit", DriverParamType_boolean, &ADMXRC5T1Smap32Bit },
  { "ADMXRC5T2MaxLCLK", DriverParamType_uint32, &ADMXRC5T2MaxLCLK },
  { "ADMXRC5T2MinLCLK", DriverParamType_uint32, &ADMXRC5T2MinLCLK },
  { "ADMXRC5T2UnlockFlash", DriverParamType_boolean, &ADMXRC5T2UnlockFlash },
  { "ADMXRC5T2Smap32Bit", DriverParamType_boolean, &ADMXRC5T2Smap32Bit },
  { "ADCPXRC4LXMaxLCLK", DriverParamType_uint32, &ADCPXRC4LXMaxLCLK },
  { "ADCPXRC4LXMinLCLK", DriverParamType_uint32, &ADCPXRC4LXMinLCLK },
  { "ADCPXRC4LXUnlockFlash", DriverParamType_boolean, &ADCPXRC4LXUnlockFlash },
  { "ADCPXRC4LXSmap32Bit", DriverParamType_boolean, &ADCPXRC4LXSmap32Bit },
  { "ADMXRC5LXAMaxLCLK", DriverParamType_uint32, &ADMXRC5LXAMaxLCLK },
  { "ADMXRC5LXAMinLCLK", DriverParamType_uint32, &ADMXRC5LXAMinLCLK },
  { "ADMXRC5LXAUnlockFlash", DriverParamType_boolean, &ADMXRC5LXAUnlockFlash },
  { "ADMXRC5LXASmap32Bit", DriverParamType_boolean, &ADMXRC5LXASmap32Bit },
  { "ADMAMC5A2MaxLCLK", DriverParamType_uint32, &ADMAMC5A2MaxLCLK },
  { "ADMAMC5A2MinLCLK", DriverParamType_uint32, &ADMAMC5A2MinLCLK },
  { "ADMAMC5A2UnlockFlash", DriverParamType_boolean, &ADMAMC5A2UnlockFlash },
  { "ADMAMC5A2Smap32Bit", DriverParamType_boolean, &ADMAMC5A2Smap32Bit },
  { "ADMXRC5TZMaxLCLK", DriverParamType_uint32, &ADMXRC5TZMaxLCLK },
  { "ADMXRC5TZMinLCLK", DriverParamType_uint32, &ADMXRC5TZMinLCLK },
  { "ADMXRC5TZUnlockFlash", DriverParamType_boolean, &ADMXRC5TZUnlockFlash },
  { "ADMXRC5TZSmap32Bit", DriverParamType_boolean, &ADMXRC5TZSmap32Bit },
  { "ADMXRC5TDA1MaxLCLK", DriverParamType_uint32, &ADMXRC5TDA1MaxLCLK },
  { "ADMXRC5TDA1MinLCLK", DriverParamType_uint32, &ADMXRC5TDA1MinLCLK },
  { "ADMXRC5TDA1UnlockFlash", DriverParamType_boolean, &ADMXRC5TDA1UnlockFlash },
  { "ADMXRC5TDA1Smap32Bit", DriverParamType_boolean, &ADMXRC5TDA1Smap32Bit },
  { NULL } /* Terminator */
};

MODULE_LICENSE("GPL");

#if (LINUX_VERSION_CODE >= 0x020600) /* For Linux 2.6.x */
module_param(PCIReadMultiple, long, S_IRUGO);
module_param(PCIWriteAndInvalidate, long, S_IRUGO);
module_param(PCI9656DmaDisable, long, S_IRUGO);
module_param(PCI9656ErrataFix, long, S_IRUGO);
module_param(PCI9656RegsAsSpace, long, S_IRUGO);
module_param(MaxDmaDesc, long, S_IRUGO);
module_param(MaxPlxDesc, long, S_IRUGO);
module_param(CommonBufferSize, long, S_IRUGO);
module_param(ICD2061PMax, long, S_IRUGO);
module_param(IRQVector, long, S_IRUGO);
module_param(ADMXRCMaxLCLK, long, S_IRUGO);
module_param(ADMXRCMinLCLK, long, S_IRUGO);
module_param(ADMXRCPMaxLCLK, long, S_IRUGO);
module_param(ADMXRCPMinLCLK, long, S_IRUGO);
module_param(ADMXRC2LMaxLCLK, long, S_IRUGO);
module_param(ADMXRC2LMinLCLK, long, S_IRUGO);
module_param(ADMXRC2MaxLCLK, long, S_IRUGO);
module_param(ADMXRC2MinLCLK, long, S_IRUGO);
module_param(ADMXPLMaxLCLK, long, S_IRUGO);
module_param(ADMXPLMinLCLK, long, S_IRUGO);
module_param(ADMXPLUnlockFlash, long, S_IRUGO);
module_param(ADPWRC2MaxLCLK, long, S_IRUGO);
module_param(ADPWRC2MinLCLK, long, S_IRUGO);
module_param(ADPDRC2MaxLCLK, long, S_IRUGO);
module_param(ADPDRC2MinLCLK, long, S_IRUGO);
module_param(ADMXPMaxLCLK, long, S_IRUGO);
module_param(ADMXPMinLCLK, long, S_IRUGO);
module_param(ADMXPUnlockFlash, long, S_IRUGO);
module_param(ADPXPIMaxLCLK, long, S_IRUGO);
module_param(ADPXPIMinLCLK, long, S_IRUGO);
module_param(ADPXPIUnlockFlash, long, S_IRUGO);
module_param(ADMXRC4LSMaxLCLK, long, S_IRUGO);
module_param(ADMXRC4LSMinLCLK, long, S_IRUGO);
module_param(ADMXRC4LSSmap32Bit, long, S_IRUGO);
module_param(ADMXRC4LXMaxLCLK, long, S_IRUGO);
module_param(ADMXRC4LXMinLCLK, long, S_IRUGO);
module_param(ADMXRC4LXSmap32Bit, long, S_IRUGO);
module_param(ADMXRC4SXMaxLCLK, long, S_IRUGO);
module_param(ADMXRC4SXMinLCLK, long, S_IRUGO);
module_param(ADMXRC4SXSmap32Bit, long, S_IRUGO);
module_param(ADMXRC4FXMaxLCLK, long, S_IRUGO);
module_param(ADMXRC4FXMinLCLK, long, S_IRUGO);
module_param(ADMXRC4FXUnlockFlash, long, S_IRUGO);
module_param(ADMXRC4FXSmap32Bit, long, S_IRUGO);
module_param(ADPEXRC4FXMaxLCLK, long, S_IRUGO);
module_param(ADPEXRC4FXMinLCLK, long, S_IRUGO);
module_param(ADPEXRC4FXUnlockFlash, long, S_IRUGO);
module_param(ADPEXRC4FXSmap32Bit, long, S_IRUGO);
module_param(ADMXRC5LXMaxLCLK, long, S_IRUGO);
module_param(ADMXRC5LXMinLCLK, long, S_IRUGO);
module_param(ADMXRC5LXUnlockFlash, long, S_IRUGO);
module_param(ADMXRC5LXSmap32Bit, long, S_IRUGO);
module_param(ADMXRC5T1MaxLCLK, long, S_IRUGO);
module_param(ADMXRC5T1MinLCLK, long, S_IRUGO);
module_param(ADMXRC5T1UnlockFlash, long, S_IRUGO);
module_param(ADMXRC5T1Smap32Bit, long, S_IRUGO);
module_param(ADMXRC5T2MaxLCLK, long, S_IRUGO);
module_param(ADMXRC5T2MinLCLK, long, S_IRUGO);
module_param(ADMXRC5T2UnlockFlash, long, S_IRUGO);
module_param(ADMXRC5T2Smap32Bit, long, S_IRUGO);
module_param(ADCPXRC4LXMaxLCLK, long, S_IRUGO);
module_param(ADCPXRC4LXMinLCLK, long, S_IRUGO);
module_param(ADCPXRC4LXUnlockFlash, long, S_IRUGO);
module_param(ADCPXRC4LXSmap32Bit, long, S_IRUGO);
module_param(ADMXRC5LXAMaxLCLK, long, S_IRUGO);
module_param(ADMXRC5LXAMinLCLK, long, S_IRUGO);
module_param(ADMXRC5LXAUnlockFlash, long, S_IRUGO);
module_param(ADMXRC5LXASmap32Bit, long, S_IRUGO);
module_param(ADMAMC5A2MaxLCLK, long, S_IRUGO);
module_param(ADMAMC5A2MinLCLK, long, S_IRUGO);
module_param(ADMAMC5A2UnlockFlash, long, S_IRUGO);
module_param(ADMAMC5A2Smap32Bit, long, S_IRUGO);
module_param(ADMXRC5TDA1MaxLCLK, long, S_IRUGO);
module_param(ADMXRC5TDA1MinLCLK, long, S_IRUGO);
module_param(ADMXRC5TDA1UnlockFlash, long, S_IRUGO);
module_param(ADMXRC5TDA1Smap32Bit, long, S_IRUGO);
#else /* For Linux 2.4.x */
MODULE_PARM(PCIReadMultiple, "l");
MODULE_PARM(PCIWriteAndInvalidate, "l");
MODULE_PARM(PCI9656DmaDisable, "l");
MODULE_PARM(PCI9656ErrataFix, "l");
MODULE_PARM(PCI9656RegsAsSpace, "l");
MODULE_PARM(MaxDmaDesc, "l");
MODULE_PARM(MaxPlxDesc, "l");
MODULE_PARM(CommonBufferSize, "l");
MODULE_PARM(ICD2061PMax, "l");
MODULE_PARM(IRQVector, "l");
MODULE_PARM(ADMXRCMaxLCLK, "l");
MODULE_PARM(ADMXRCMinLCLK, "l");
MODULE_PARM(ADMXRCPMaxLCLK, "l");
MODULE_PARM(ADMXRCPMinLCLK, "l");
MODULE_PARM(ADMXRC2LMaxLCLK, "l");
MODULE_PARM(ADMXRC2LMinLCLK, "l");
MODULE_PARM(ADMXRC2MaxLCLK, "l");
MODULE_PARM(ADMXRC2MinLCLK, "l");
MODULE_PARM(ADMXPLMaxLCLK, "l");
MODULE_PARM(ADMXPLMinLCLK, "l");
MODULE_PARM(ADMXPLUnlockFlash, "l");
MODULE_PARM(ADPWRC2MaxLCLK, "l");
MODULE_PARM(ADPWRC2MinLCLK, "l");
MODULE_PARM(ADPDRC2MaxLCLK, "l");
MODULE_PARM(ADPDRC2MinLCLK, "l");
MODULE_PARM(ADMXPMaxLCLK, "l");
MODULE_PARM(ADMXPMinLCLK, "l");
MODULE_PARM(ADMXPUnlockFlash, "l");
MODULE_PARM(ADPXPIMaxLCLK, "l");
MODULE_PARM(ADPXPIMinLCLK, "l");
MODULE_PARM(ADPXPIUnlockFlash, "l");
MODULE_PARM(ADMXRC4LSMaxLCLK, "l");
MODULE_PARM(ADMXRC4LSMinLCLK, "l");
MODULE_PARM(ADMXRC4LSSmap32Bit, "l");
MODULE_PARM(ADMXRC4LXMaxLCLK, "l");
MODULE_PARM(ADMXRC4LXMinLCLK, "l");
MODULE_PARM(ADMXRC4LXSmap32Bit, "l");
MODULE_PARM(ADMXRC4SXMaxLCLK, "l");
MODULE_PARM(ADMXRC4SXMinLCLK, "l");
MODULE_PARM(ADMXRC4SXSmap32Bit, "l");
MODULE_PARM(ADMXRC4FXMaxLCLK, "l");
MODULE_PARM(ADMXRC4FXMinLCLK, "l");
MODULE_PARM(ADMXRC4FXUnlockFlash, "l");
MODULE_PARM(ADMXRC4FXSmap32Bit, "l");
MODULE_PARM(ADPEXRC4FXMaxLCLK, "l");
MODULE_PARM(ADPEXRC4FXMinLCLK, "l");
MODULE_PARM(ADPEXRC4FXUnlockFlash, "l");
MODULE_PARM(ADPEXRC4FXSmap32Bit, "l");
MODULE_PARM(ADMXRC5LXMaxLCLK, "l");
MODULE_PARM(ADMXRC5LXMinLCLK, "l");
MODULE_PARM(ADMXRC5LXUnlockFlash, "l");
MODULE_PARM(ADMXRC5LXSmap32Bit, "l");
MODULE_PARM(ADMXRC5T1MaxLCLK, "l");
MODULE_PARM(ADMXRC5T1MinLCLK, "l");
MODULE_PARM(ADMXRC5T1UnlockFlash, "l");
MODULE_PARM(ADMXRC5T1Smap32Bit, "l");
MODULE_PARM(ADMXRC5T2MaxLCLK, "l");
MODULE_PARM(ADMXRC5T2MinLCLK, "l");
MODULE_PARM(ADMXRC5T2UnlockFlash, "l");
MODULE_PARM(ADMXRC5T2Smap32Bit, "l");
MODULE_PARM(ADCPXRC4LXMaxLCLK, "l");
MODULE_PARM(ADCPXRC4LXMinLCLK, "l");
MODULE_PARM(ADCPXRC4LXUnlockFlash, "l");
MODULE_PARM(ADCPXRC4LXSmap32Bit, "l");
MODULE_PARM(ADMXRC5LXAMaxLCLK, "l");
MODULE_PARM(ADMXRC5LXAMinLCLK, "l");
MODULE_PARM(ADMXRC5LXAUnlockFlash, "l");
MODULE_PARM(ADMXRC5LXASmap32Bit, "l");
MODULE_PARM(ADMAMC5A2MaxLCLK, "l");
MODULE_PARM(ADMAMC5A2MinLCLK, "l");
MODULE_PARM(ADMAMC5A2UnlockFlash, "l");
MODULE_PARM(ADMAMC5A2Smap32Bit, "l");
MODULE_PARM(ADMXRC5TZMaxLCLK, "l");
MODULE_PARM(ADMXRC5TZMinLCLK, "l");
MODULE_PARM(ADMXRC5TZUnlockFlash, "l");
MODULE_PARM(ADMXRC5TZSmap32Bit, "l");
MODULE_PARM(ADMXRC5TDA1MaxLCLK, "l");
MODULE_PARM(ADMXRC5TDA1MinLCLK, "l");
MODULE_PARM(ADMXRC5TDA1UnlockFlash, "l");
MODULE_PARM(ADMXRC5TDA1Smap32Bit, "l");
#endif

static int
charlower(int c)
{
  if (c >= 'A' && c <= 'Z') {
    return c - 'A' + 'a';
  } else {
    return c;
  }
}

static os_boolean
strNoCaseCmp(
  const char* s1,
  const char* s2)
{
  char a, b;

  while (1) {
    a = charlower(*s1++);
    b = charlower(*s2++);
    if (a != b) {
      return FALSE;
    } else {
      if (a == '\0') {
	break;
      }
    }
  }
  return TRUE;
}

/*
** -----------------------------------------------------------------------
** Exported routines
** -----------------------------------------------------------------------
*/

os_boolean
os_get_driver_param(
  OS_DRIVER_OBJECT* pDrvObj,
  const char*       mnemonic,
  DriverParamType   type,
  void*             value,
  unsigned long     size)
{
  unsigned int i;

  for (i = 0; module_params[i].mnemonic != NULL; i++) {
    if (strNoCaseCmp(module_params[i].mnemonic, mnemonic) && module_params[i].type == type) {
      switch (type) {
      case DriverParamType_uint32:
	*((os_uint32*) value) = (os_uint32) *((unsigned long*) module_params[i].value);
	break;
	
      case DriverParamType_int32:
	*((os_int32*) value) = (os_int32) *((long*) module_params[i].value);
	break;
	
      case DriverParamType_boolean:
	*((os_boolean*) value) = (os_boolean) *((unsigned long*) module_params[i].value);
	break;
	
      default:
	break;
      }
      break;
    }
  }

  return FALSE;
}
