#ifndef _ADATA_METHODS_H
#define _ADATA_METHODS_H

#include "device.h"

/*
** The error method; should never be called if things are working correctly.
*/
extern MethodResult
methodError(
  DeviceContext* pDevCtx, ...);

/*
** MethodFlash methods
*/

#define methodFlashNotPresent methodFlashXrc2l

extern MethodResult
methodFlashXrc2l(
  DeviceContext* pDevCtx,
  os_boolean     write,
  os_uint32      location,
  os_uint16*     value);

extern MethodResult
methodFlashPlxbased(
  DeviceContext* pDevCtx,
  os_boolean     write,
  os_uint32      location,
  os_uint16*     value);

extern MethodResult
methodFlashXpl(
  DeviceContext* pDevCtx,
  os_boolean     write,
  os_uint32      location,
  os_uint16*     value);

extern MethodResult
methodFlashXrc2n(
  DeviceContext* pDevCtx,
  os_boolean     write,
  os_uint32      location,
  os_uint16*     value);

extern MethodResult
methodFlashXpl(
  DeviceContext* pDevCtx,
  os_boolean     write,
  os_uint32      location,
  os_uint16*     value);

extern MethodResult
methodFlashXprev0(
  DeviceContext* pDevCtx,
  os_boolean     write,
  os_uint32      location,
  os_uint16*     value);

extern MethodResult
methodFlashXprev1(
  DeviceContext* pDevCtx,
  os_boolean     write,
  os_uint32      location,
  os_uint16*     value);

extern MethodResult
methodFlashXrc4ls(
  DeviceContext* pDevCtx,
  os_boolean     write,
  os_uint32      location,
  os_uint16*     value);

extern MethodResult
methodFlashXrce4fx(
  DeviceContext* pDevCtx,
  os_boolean     write,
  os_uint32      location,
  os_uint16*     value);

extern MethodResult
methodFlashXrc4fx(
  DeviceContext* pDevCtx,
  os_boolean     write,
  os_uint32      location,
  os_uint16*     value);

extern MethodResult
methodFlashBbp(
  DeviceContext* pDevCtx,
  os_boolean     write,
  os_uint32      location,
  os_uint16*     value);

/*
** MethodFlashReset methods
*/

/* PLX version of Flash reset method does nothing */
#define methodFlashResetNotPresent methodFlashResetPlxbased

extern MethodResult
methodFlashResetPlxbased(
  DeviceContext* pDevCtx);

extern MethodResult
methodFlashResetXpl(
  DeviceContext* pDevCtx);

extern MethodResult
methodFlashResetXp(
  DeviceContext* pDevCtx);

extern MethodResult
methodFlashResetXrc4ls(
  DeviceContext* pDevCtx);

extern MethodResult
methodFlashResetXrce4fx(
  DeviceContext* pDevCtx);

extern MethodResult
methodFlashResetXrc4fx(
  DeviceContext* pDevCtx);

extern MethodResult
methodFlashResetBbp(
  DeviceContext* pDevCtx);

/*
** MethodConfigMem
*/

extern MethodResult
methodConfigMemPlxbased(
  DeviceContext* pDevCtx,
  os_boolean     write,
  unsigned long  location,
  os_uint32*     value);

extern MethodResult
methodConfigMemXpl(
  DeviceContext* pDevCtx,
  os_boolean     write,
  unsigned long  location,
  os_uint32*     value);

extern MethodResult
methodConfigMemXrc5t1VpdBram(
  DeviceContext* pDevCtx,
  os_boolean     write,
  unsigned long  location,
  os_uint32*     value);

extern MethodResult
methodConfigMemAmc5a2(
  DeviceContext* pDevCtx,
  os_boolean     write,
  unsigned long  location,
  os_uint32*     value);

extern MethodResult
methodConfigMemBbp(
  DeviceContext* pDevCtx,
  os_boolean     write,
  unsigned long  location,
  os_uint32*     value);

/*
** MethodIntEnab
*/

extern MethodResult
methodIntEnabPlxbased(
  DeviceContext* pDevCtx,
  os_boolean     enable);

extern MethodResult
methodIntEnabXplrev0(
  DeviceContext* pDevCtx,
  os_boolean     enable);

extern MethodResult
methodIntEnabXplrev1(
  DeviceContext* pDevCtx,
  os_boolean     enable);

extern MethodResult
methodIntEnabXp(
  DeviceContext* pDevCtx,
  os_boolean     enable);

extern MethodResult
methodIntEnabXrc4ls(
  DeviceContext* pDevCtx,
  os_boolean     enable);

extern MethodResult
methodIntEnabXrce4fx(
  DeviceContext* pDevCtx,
  os_boolean     enable);

extern MethodResult
methodIntEnabXrc4fx(
  DeviceContext* pDevCtx,
  os_boolean     enable);

extern MethodResult
methodIntEnabBbp(
  DeviceContext* pDevCtx,
  os_boolean     enable);

/*
** MethodIsr
*/

extern os_boolean
methodIsrPlxbased(
  DeviceContext* pDevCtx);

extern os_boolean
methodIsrXpl(
  DeviceContext* pDevCtx);

extern os_boolean
methodIsrXp(
  DeviceContext* pDevCtx);

extern os_boolean
methodIsrXrc4ls(
  DeviceContext* pDevCtx);

extern os_boolean
methodIsrXrce4fx(
  DeviceContext* pDevCtx);

extern os_boolean
methodIsrXrc4fx(
  DeviceContext* pDevCtx);

extern os_boolean
methodIsrBbp(
  DeviceContext* pDevCtx);

/*
** MethodFpga
*/

extern MethodResult
methodFpgaPlxbased(
  DeviceContext* pDevCtx,
  unsigned int   index,
  MethodFpgaOp   op,
  void*          operand);

extern MethodResult
methodFpgaXpl(
  DeviceContext* pDevCtx,
  unsigned int   index,
  MethodFpgaOp   op,
  void*          operand);

extern MethodResult
methodFpgaXrc4ls(
  DeviceContext* pDevCtx,
  unsigned int   index,
  MethodFpgaOp   op,
  void*          operand);

extern MethodResult
methodFpgaXrce4fx(
  DeviceContext* pDevCtx,
  unsigned int   index,
  MethodFpgaOp   op,
  void*          operand);

extern MethodResult
methodFpgaXrc4fx(
  DeviceContext* pDevCtx,
  unsigned int   index,
  MethodFpgaOp   op,
  void*          operand);

extern MethodResult
methodFpgaBbp(
  DeviceContext* pDevCtx,
  unsigned int   index,
  MethodFpgaOp   op,
  void*          operand);

/*
** MethodSelectMap
*/

extern MethodResult
methodSelectMapPlxbased(
  DeviceContext* pDevCtx,
  unsigned int   index,
  os_boolean     write,
  os_uint8*      buffer,
  unsigned long  length);

extern MethodResult
methodSelectMapXpl(
  DeviceContext* pDevCtx,
  unsigned int   index,
  os_boolean     write,
  os_uint8*      buffer,
  unsigned long  length);

extern MethodResult
methodSelectMapXrc4ls(
  DeviceContext* pDevCtx,
  unsigned int   index,
  os_boolean     write,
  os_uint8*      buffer,
  unsigned long  length);

extern MethodResult
methodSelectMapXrc4lx(
  DeviceContext* pDevCtx,
  unsigned int   index,
  os_boolean     write,
  os_uint8*      buffer,
  unsigned long  length);

extern MethodResult
methodSelectMapXrc4sx(
  DeviceContext* pDevCtx,
  unsigned int   index,
  os_boolean     write,
  os_uint8*      buffer,
  unsigned long  length);

extern MethodResult
methodSelectMapXrce4fx(
  DeviceContext* pDevCtx,
  unsigned int   index,
  os_boolean     write,
  os_uint8*      buffer,
  unsigned long  length);

extern MethodResult
methodSelectMapXrc4fx(
  DeviceContext* pDevCtx,
  unsigned int   index,
  os_boolean     write,
  os_uint8*      buffer,
  unsigned long  length);

extern MethodResult
methodSelectMapBbp(
  DeviceContext* pDevCtx,
  unsigned int   index,
  os_boolean     write,
  os_uint8*      buffer,
  unsigned long  length);

/*
** MethodConfigSpace
*/

extern MethodResult
methodConfigSpacePlxbased(
  DeviceContext* pDevCtx,
  os_boolean     write,
  unsigned int   space,
  os_uint32*     value);

extern MethodResult
methodConfigSpaceXplrev0(
  DeviceContext* pDevCtx,
  os_boolean     write,
  unsigned int   space,
  os_uint32*     value);

extern MethodResult
methodConfigSpaceXplrev1(
  DeviceContext* pDevCtx,
  os_boolean     write,
  unsigned int   space,
  os_uint32*     value);

/*
** MethodAddDmaBuffer
*/

extern MethodResult
methodAddDmaBufferPlxbased(
  DeviceContext*  pDevCtx,
  unsigned int    channel,
  unsigned int    index,
  os_boolean      bDmaWrite,
  DmaMapping*     pMapping);

extern MethodResult
methodAddDmaBufferXplrev0(
  DeviceContext*  pDevCtx,
  unsigned int    channel,
  unsigned int    index,
  os_boolean      bDmaWrite,
  DmaMapping*     pMapping);

extern MethodResult
methodAddDmaBufferXplrev1(
  DeviceContext*  pDevCtx,
  unsigned int    channel,
  unsigned int    index,
  os_boolean      bDmaWrite,
  DmaMapping*     pMapping);

/*
** MethodDoDma
*/

extern MethodResult
methodDoDmaXrc(
  DeviceContext*  pDevCtx,
  unsigned int    channel,
  os_boolean      start,
  os_uint32       mode,
  unsigned int    numBuffer);

extern MethodResult
methodDoDmaXplrev0(
  DeviceContext*  pDevCtx,
  unsigned int    channel,
  os_boolean      start,
  os_uint32       mode,
  unsigned int    numBuffer);

extern MethodResult
methodDoDmaXplrev1(
  DeviceContext*  pDevCtx,
  unsigned int    channel,
  os_boolean      start,
  os_uint32       mode,
  unsigned int    numBuffer);

extern MethodResult
methodDoDmaXrc4ls(
  DeviceContext*  pDevCtx,
  unsigned int    channel,
  os_boolean      start,
  os_uint32       mode,
  unsigned int    numBuffer);

extern MethodResult
methodDoDmaXrce4fx(
  DeviceContext*  pDevCtx,
  unsigned int    channel,
  os_boolean      start,
  os_uint32       mode,
  unsigned int    numBuffer);

extern MethodResult
methodDoDmaXrc4fx(
  DeviceContext*  pDevCtx,
  unsigned int    channel,
  os_boolean      start,
  os_uint32       mode,
  unsigned int    numBuffer);

extern MethodResult
methodDoDmaBbp(
  DeviceContext*  pDevCtx,
  unsigned int    channel,
  os_boolean      start,
  os_uint32       mode,
  unsigned int    numBuffer);

/*
** MethodClockWord
*/

extern MethodResult
methodClockWordXrc(
  DeviceContext* pDevCtx,
  unsigned int   clockIndex,
  os_uint32      freq,
  os_uint32*     actual,
  os_uint32*     clockWord);

extern MethodResult
methodClockWordXrc2(
  DeviceContext* pDevCtx,
  unsigned int   clockIndex,
  os_uint32      freq,
  os_uint32*     actual,
  os_uint32*     clockWord);

extern MethodResult
methodClockWordXpl(
  DeviceContext* pDevCtx,
  unsigned int   clockIndex,
  os_uint32      freq,
  os_uint32*     actual,
  os_uint32*     clockWord);

extern MethodResult
methodClockWordXrc4ls(
  DeviceContext* pDevCtx,
  unsigned int   clockIndex,
  os_uint32      freq,
  os_uint32*     actual,
  os_uint32*     clockWord);

extern MethodResult
methodClockWordXrce4fx(
  DeviceContext* pDevCtx,
  unsigned int   clockIndex,
  os_uint32      freq,
  os_uint32*     actual,
  os_uint32*     clockWord);

extern MethodResult
methodClockWordXrc4fx(
  DeviceContext* pDevCtx,
  unsigned int   clockIndex,
  os_uint32      freq,
  os_uint32*     actual,
  os_uint32*     clockWord);

extern MethodResult
methodClockWordXrc5lx(
  DeviceContext* pDevCtx,
  unsigned int   clockIndex,
  os_uint32      freq,
  os_uint32*     actual,
  os_uint32*     clockWord);

extern MethodResult
methodClockWordXrc5lxa(
  DeviceContext* pDevCtx,
  unsigned int   clockIndex,
  os_uint32      freq,
  os_uint32*     actual,
  os_uint32*     clockWord);

extern MethodResult
methodClockWordXrcp4lx(
  DeviceContext* pDevCtx,
  unsigned int   clockIndex,
  os_uint32      freq,
  os_uint32*     actual,
  os_uint32*     clockWord);

extern MethodResult
methodClockWordBbp(
  DeviceContext* pDevCtx,
  unsigned int   clockIndex,
  os_uint32      freq,
  os_uint32*     actual,
  os_uint32*     clockWord);

/*
** MethodClockProg
*/

extern MethodResult
methodClockProgPlxbased(
  DeviceContext* pDevCtx,
  unsigned int   index,
  os_uint32      freq,
  os_uint32      clockWord,
  os_boolean     bBefore);

extern MethodResult
methodClockProgXplrev0(
  DeviceContext* pDevCtx,
  unsigned int   index,
  os_uint32      freq,
  os_uint32      clockWord,
  os_boolean     bBefore);

extern MethodResult
methodClockProgXplrev2(
  DeviceContext* pDevCtx,
  unsigned int   index,
  os_uint32      freq,
  os_uint32      clockWord,
  os_boolean     bBefore);

extern MethodResult
methodClockProgXrc4ls(
  DeviceContext* pDevCtx,
  unsigned int   index,
  os_uint32      freq,
  os_uint32      clockWord,
  os_boolean     bBefore);

extern MethodResult
methodClockProgXrce4fx(
  DeviceContext* pDevCtx,
  unsigned int   index,
  os_uint32      freq,
  os_uint32      clockWord,
  os_boolean     bBefore);

extern MethodResult
methodClockProgXrc4fx(
  DeviceContext* pDevCtx,
  unsigned int   index,
  os_uint32      freq,
  os_uint32      clockWord,
  os_boolean     bBefore);

extern MethodResult
methodClockProgXrc5lx(
  DeviceContext* pDevCtx,
  unsigned int   index,
  os_uint32      freq,
  os_uint32      clockWord,
  os_boolean     bBefore);

extern MethodResult
methodClockProgXrc5lxa(
  DeviceContext* pDevCtx,
  unsigned int   index,
  os_uint32      freq,
  os_uint32      clockWord,
  os_boolean     bBefore);

extern MethodResult
methodClockProgXrcp4lx(
  DeviceContext* pDevCtx,
  unsigned int   index,
  os_uint32      freq,
  os_uint32      clockWord,
  os_boolean     bBefore);

extern MethodResult
methodClockProgBbp(
  DeviceContext* pDevCtx,
  unsigned int   index,
  os_uint32      freq,
  os_uint32      clockWord,
  os_boolean     bBefore);

/*
** MethodInitCardInfo methods
*/

extern MethodResult
methodInitCardInfoXrc(
  DeviceContext* pDevCtx);

extern MethodResult
methodInitCardInfoXrc2(
  DeviceContext* pDevCtx);

extern MethodResult
methodInitCardInfoXpl(
  DeviceContext* pDevCtx);

extern MethodResult
methodInitCardInfoWrc2(
  DeviceContext* pDevCtx);

extern MethodResult
methodInitCardInfoDrc2(
  DeviceContext* pDevCtx);

extern MethodResult
methodInitCardInfoXp(
  DeviceContext* pDevCtx);

extern MethodResult
methodInitCardInfoXpi(
  DeviceContext* pDevCtx);

extern MethodResult
methodInitCardInfoXrc4ls(
  DeviceContext* pDevCtx);

extern MethodResult
methodInitCardInfoXrc4lx(
  DeviceContext* pDevCtx);

extern MethodResult
methodInitCardInfoXrc4sx(
  DeviceContext* pDevCtx);

extern MethodResult
methodInitCardInfoXrc4fx(
  DeviceContext* pDevCtx);

extern MethodResult
methodInitCardInfoXrce4fx(
  DeviceContext* pDevCtx);

extern MethodResult
methodInitCardInfoXrc5lx(
  DeviceContext* pDevCtx);

extern MethodResult
methodInitCardInfoXrc5t1(
  DeviceContext* pDevCtx);

extern MethodResult
methodInitCardInfoXrc5t1VpdBram(
  DeviceContext* pDevCtx);

extern MethodResult
methodInitCardInfoXrc5t2(
  DeviceContext* pDevCtx);

extern MethodResult
methodInitCardInfoXrc5lxa(
  DeviceContext* pDevCtx);

extern MethodResult
methodInitCardInfoXrcp4lx(
  DeviceContext* pDevCtx);

extern MethodResult
methodInitCardInfoAmc5a2(
  DeviceContext* pDevCtx);

extern MethodResult
methodInitCardInfoXrc5tz(
  DeviceContext* pDevCtx);

extern MethodResult
methodInitCardInfoBbp(
  DeviceContext* pDevCtx);

extern MethodResult
methodInitCardInfoXrc5tda1(
  DeviceContext* pDevCtx);

/*
** MethodLocalBusReset methods
*/

MethodResult
methodLocalBusReset9080(
  DeviceContext* pDevCtx,
  os_boolean bAssert);

MethodResult
methodLocalBusReset9656(
  DeviceContext* pDevCtx,
  os_boolean bAssert);

MethodResult
methodLocalBusResetXpl(
  DeviceContext* pDevCtx,
  os_boolean bAssert);

/*
** MethodHwInit methods
*/

MethodResult
methodHwInitPci9080based(
  DeviceContext* pDevCtx);

MethodResult
methodHwInitPci9656based(
  DeviceContext* pDevCtx);

MethodResult
methodHwInitXplrev0(
  DeviceContext* pDevCtx);

MethodResult
methodHwInitXplrev1(
  DeviceContext* pDevCtx);

MethodResult
methodHwInitXp(
  DeviceContext* pDevCtx);

MethodResult
methodHwInitXrc4ls(
  DeviceContext* pDevCtx);

MethodResult
methodHwInitXrc4fx(
  DeviceContext* pDevCtx);

MethodResult
methodHwInitXrce4fx(
  DeviceContext* pDevCtx);

MethodResult
methodHwInitBbp(
  DeviceContext* pDevCtx);

#endif



