/*
** ADM-XRC Series Driver
**
** (C) Copright 2001-2005 Alpha Data
**
** methods_fpga.c - Model-specific routines for configuring target FPGA.
*/

#include "drvparam.h"
#include "flash.h"
#include "methods.h"

MethodResult
methodFpgaPlxbased(
  DeviceContext* pDevCtx,
  unsigned int index, /* ignored */
  MethodFpgaOp op,
  void* operand)
{
  CONTROLXRC* pRegs = pDevCtx->hw.control.plxbased;
  OS_BAR_HANDLE handle = pDevCtx->hw.controlHandle;
  os_uint8 fcon;
  
  switch (op) {
  case MethodFpgaOp_SetProg:
    {
      os_boolean asserted = *((os_boolean*) operand);
      
      fcon = os_read_register_le8(handle, &pRegs->FCON) & ~0x6U;
      if (asserted) {
	fcon |= 0x1U;
      } else {
	fcon &= (os_uint8) ~0x1U;
      }
      os_write_register_le8(handle, &pRegs->FCON, fcon);
      break;
    }
    
  case MethodFpgaOp_GetInit:
    {
      fcon = os_read_register_le8(handle, &pRegs->FCON);
      *((os_boolean*) operand) = (fcon & 0x2U) != 0;
      break;
    }
    
  case MethodFpgaOp_GetDone:
    {
      fcon = os_read_register_le8(handle, &pRegs->FCON);
      *((os_boolean*) operand) = (fcon & 0x4U) != 0;
      break;
    }
    
  default:
    return MethodResult_NotSupported;
  }
  
  return MethodResult_Success;
}

MethodResult
methodFpgaXpl(
  DeviceContext* pDevCtx,
  unsigned int index, /* ignored */
  MethodFpgaOp op,
  void* operand)
{
  XPLSPACE1REGS* pRegs = pDevCtx->hw.control.xpl;
  OS_BAR_HANDLE handle = pDevCtx->hw.controlHandle;
  os_uint32 fpctl;

  switch (op) {
  case MethodFpgaOp_SetProg:
    {
      os_boolean bAsserted = *((os_boolean*) operand);
      
      fpctl = os_read_register_le32(handle, &pRegs->FPCTL) & ~0x6U;
      if (bAsserted) {
	fpctl |= 0x1U;
      } else {
	fpctl &= ~0x1U;
      }
      os_write_register_le32(handle, &pRegs->FPCTL, fpctl);
      break;
    }
    
  case MethodFpgaOp_GetInit:
    {
      fpctl = os_read_register_le32(handle, &pRegs->FPCTL);
      *((os_boolean*) operand) = (fpctl & 0x2U) != 0;
      break;
    }
    
  case MethodFpgaOp_GetDone:
    {
      fpctl = os_read_register_le32(handle, &pRegs->FPCTL);
      *((os_boolean*) operand) = (fpctl & 0x4U) != 0;
      break;
    }
    
  default:
    return MethodResult_NotSupported;
  }
  
  return MethodResult_Success;
}

MethodResult
methodFpgaXrc4ls(
  DeviceContext* pDevCtx,
  unsigned int index, /* ignored */
  MethodFpgaOp op,
  void* operand)
{
  CONTROLXRC4LS* pRegs = pDevCtx->hw.control.xrc4ls;
  OS_BAR_HANDLE handle = pDevCtx->hw.controlHandle;
  os_uint32 fpctl;
  os_boolean bAssert;

  switch (op) {
  case MethodFpgaOp_SetProg:
    bAssert = *((os_boolean*) operand);
    if (bAssert) {
      pDevCtx->shadow.regs.xrc4ls.fpctl |= 0x1U;
    } else {
      pDevCtx->shadow.regs.xrc4ls.fpctl &= ~0x1U;
    }
    os_write_register_le32(handle, &pRegs->FPCTL, pDevCtx->shadow.regs.xrc4ls.fpctl);
    os_read_register_le32(handle, &pRegs->FPCTL);
    break;
    
  case MethodFpgaOp_GetInit:
    fpctl = os_read_register_le32(handle, &pRegs->FPCTL);
    *((os_boolean*) operand) = (fpctl & 0x2U) != 0;
    os_dbgprint(6, ("GetInit: fpctl=0x%lx\n", (unsigned long)fpctl));
    break;
    
  case MethodFpgaOp_GetDone:
    fpctl = os_read_register_le32(handle, &pRegs->FPCTL);
    *((os_boolean*) operand) = (fpctl & 0x4U) != 0;
    os_dbgprint(6, ("GetDone: fpctl=0x%lx\n", (unsigned long)fpctl));
    break;
    
  default:
    return MethodResult_NotSupported;
  }

  return MethodResult_Success;
}

MethodResult
methodFpgaXrc4fx(
  DeviceContext* pDevCtx,
  unsigned int index, /* ignored */
  MethodFpgaOp op,
  void* operand)
{
  XRC4FXSPACE1REGS* pRegs = pDevCtx->hw.control.xrc4fx;
  OS_BAR_HANDLE handle = pDevCtx->hw.controlHandle;
  os_uint32 fpctl;
  os_boolean bAssert;

  switch (op) {
  case MethodFpgaOp_SetProg:
    bAssert = *((os_boolean*) operand);
    if (bAssert) {
      pDevCtx->shadow.regs.xrc4fx.fpctl |= 0x1U;
    } else {
      pDevCtx->shadow.regs.xrc4fx.fpctl &= ~0x1U;
    }
    os_write_register_le32(handle, &pRegs->FPCTL, pDevCtx->shadow.regs.xrc4fx.fpctl);
    os_read_register_le32(handle, &pRegs->FPCTL);
    break;
    
  case MethodFpgaOp_GetInit:
    fpctl = os_read_register_le32(handle, &pRegs->FPCTL);
    *((os_boolean*) operand) = (fpctl & 0x2U) != 0;
    break;
    
  case MethodFpgaOp_GetDone:
    fpctl = os_read_register_le32(handle, &pRegs->FPCTL);
    *((os_boolean*) operand) = (fpctl & 0x4U) != 0;
    break;
    
  default:
    return MethodResult_NotSupported;
  }
  
  return MethodResult_Success;
}

MethodResult
methodFpgaXrce4fx(
  DeviceContext* pDevCtx,
  unsigned int index, /* ignored */
  MethodFpgaOp op,
  void* operand)
{
  XRCE4FXSPACE1REGS* pRegs = pDevCtx->hw.control.xrce4fx;
  OS_BAR_HANDLE handle = pDevCtx->hw.controlHandle;
  os_uint32 fpctl;
  
  switch (op) {
  case MethodFpgaOp_SetProg:
    {
      os_boolean	bAssert = *((os_boolean*) operand);
      
      if (bAssert) {
	pDevCtx->shadow.regs.xrce4fx.fpctl |= 0x1U;
      } else {
	pDevCtx->shadow.regs.xrce4fx.fpctl &= ~0x1U;
      }
      os_write_register_le32(handle, &pRegs->FPCTL, pDevCtx->shadow.regs.xrce4fx.fpctl);
      os_read_register_le32(handle, &pRegs->FPCTL);
      break;
    }
    
  case MethodFpgaOp_GetInit:
    fpctl = os_read_register_le32(handle, &pRegs->FPCTL);
    *((os_boolean*) operand) = (fpctl & 0x2U) != 0;
    break;
    
  case MethodFpgaOp_GetDone:
    fpctl = os_read_register_le32(handle, &pRegs->FPCTL);
    *((os_boolean*) operand) = (fpctl & 0x4U) != 0;
    break;
    
  default:
    return MethodResult_NotSupported;
  }
  
  return MethodResult_Success;
}

MethodResult
methodFpgaBbp(
  DeviceContext* pDevCtx,
  unsigned int index,
  MethodFpgaOp op,
  void* operand)
{
  BBPSPACE1REGS* pRegs = pDevCtx->hw.control.bbp;
  OS_BAR_HANDLE handle = pDevCtx->hw.controlHandle;
  os_uint32 fpctl;
  os_boolean bAssert;
  os_uint32* pFpctl;

  if (index == 0) {
    pFpctl = &pRegs->FPCTL0;
  } else {
    pFpctl = &pRegs->FPCTL1;
  }

  switch (op) {
  case MethodFpgaOp_SetProg:
    bAssert = *((os_boolean*) operand);
    if (bAssert) {
      pDevCtx->shadow.regs.bbp.fpctl[index] |= 0x1U;
    } else {
      pDevCtx->shadow.regs.bbp.fpctl[index] &= ~0x1U;
    }
    os_write_register_le32(handle, pFpctl, pDevCtx->shadow.regs.bbp.fpctl[index]);
    os_read_register_le32(handle, pFpctl);
    break;
    
  case MethodFpgaOp_GetInit:
    fpctl = os_read_register_le32(handle, pFpctl);
    *((os_boolean*) operand) = (fpctl & 0x2U) != 0;
    break;
    
  case MethodFpgaOp_GetDone:
    fpctl = os_read_register_le32(handle, pFpctl);
    *((os_boolean*) operand) = (fpctl & 0x4U) != 0;
    break;
    
  default:
    return MethodResult_NotSupported;
  }
  
  return MethodResult_Success;
}

/*
** -----------------------------------------------------------------------
** MethodSelectMap
** -----------------------------------------------------------------------
*/

MethodResult
methodSelectMapPlxbased(
  DeviceContext* pDevCtx,
  unsigned int index, /* ignored */
  os_boolean write,
  os_uint8* buffer,
  unsigned long length)
{
  CONTROLXRC* pRegs = pDevCtx->hw.control.plxbased;
  OS_BAR_HANDLE handle = pDevCtx->hw.controlHandle;

  if (write) {
    while (length--) {
      os_write_register_le8(handle, &pRegs->SELECTMAP[0], *buffer++);
    }
  } else {
    while (length--) {
      *buffer++ = os_read_register_le8(handle, &pRegs->SELECTMAP[0]);
    }
  }
  
  return MethodResult_Success;
}

MethodResult
methodSelectMapXpl(
  DeviceContext* pDevCtx,
  unsigned int index, /* ignored */
  os_boolean write,
  os_uint8* buffer,
  unsigned long length)
{
  XPLSPACE1REGS* pRegs = pDevCtx->hw.control.xpl;
  OS_BAR_HANDLE handle = pDevCtx->hw.controlHandle;
  unsigned long n8, n32;
  os_uint32 val32;

  n8 = length & 0x3U;
  n32 = length >> 2;
  if (write) {
    while (n32--) {
      val32 = (*buffer++ << 0);
      val32 |= (*buffer++ << 8);
      val32 |= (*buffer++ << 16);
      val32 |= (*buffer++ << 24);
      os_write_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0], val32);
    }
    while (n8--) {
      os_write_register_le8(handle, &pRegs->FPSMAP[0], *buffer++);
    }
  } else {
    while (n32--) {
      val32 = os_read_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0]);
      *buffer++ = (os_uint8) (val32 >> 0);
      *buffer++ = (os_uint8) (val32 >> 8);
      *buffer++ = (os_uint8) (val32 >> 16);
      *buffer++ = (os_uint8) (val32 >> 24);
    }
    while (n8--) {
      *buffer++ = os_read_register_le8(handle, &pRegs->FPSMAP[0]);
    }
  }

  return MethodResult_Success;
}

MethodResult
methodSelectMapXrc4ls(
  DeviceContext* pDevCtx,
  unsigned int index, /* ignored */
  os_boolean write,
  os_uint8* buffer,
  unsigned long length)
{
  CONTROLXRC4LS* pRegs = pDevCtx->hw.control.xrc4ls;
  OS_BAR_HANDLE handle = pDevCtx->hw.controlHandle;
  const os_boolean bByteSwap = FALSE;
  unsigned long i, n8, n32;
  os_uint32 val32;

  if (admxrcDriverParams.model.xrc4ls.smap32Bit) {
    n8 = length & 0x3U;
    n32 = length >> 2;
    if (write) {
      while (n32--) {
	if (bByteSwap) {
	  val32 = (*buffer++ << 24);
	  val32 |= (*buffer++ << 16);
	  val32 |= (*buffer++ << 8);
	  val32 |= (*buffer++ << 0);
	} else {
	  val32 = (*buffer++ << 0);
	  val32 |= (*buffer++ << 8);
	  val32 |= (*buffer++ << 16);
	  val32 |= (*buffer++ << 24);
	}
	os_write_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0], val32);
      }
      if (n8) {
	val32 = 0x0;
	if (bByteSwap) {
	  for (i = 0; i < n8; i++) {
	    val32 = (val32 << 8) | *buffer++;
	  }
	  for (; i < 4; i++) {
	    val32 = (val32 << 8) | 0xffU;
	  }
	} else {
	  for (i = 0; i < n8; i++) {
	    val32 = (val32 >> 8) | (*buffer++ << 24);
	  }
	  for (; i < 4; i++) {
	    val32 = (val32 >> 8) | 0xff000000U;
	  }
	}
	os_write_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0], val32);
      }
      os_read_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0]);
    } else {
      while (n32--) {
	val32 = os_read_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0]);
	*buffer++ = (os_uint8) (val32 >> 0);
	*buffer++ = (os_uint8) (val32 >> 8);
	*buffer++ = (os_uint8) (val32 >> 16);
	*buffer++ = (os_uint8) (val32 >> 24);
      }
      val32 = os_read_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0]);
      while (n8--) {
	*buffer++ = (os_uint8) val32;
	val32 >>= 8;
      }
    }
  } else {
    if (write) {
      while (length--) {
	val32 = (*buffer++ << 24);
	os_write_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0], val32);
      }
      os_read_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0]);
    } else {
      while (length--) {
	val32 = os_read_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0]);
	*buffer++ = (os_uint8) (val32 >> 24);
      }
      val32 = os_read_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0]);
    }
  }

  return MethodResult_Success;
}

MethodResult
methodSelectMapXrc4lx(
  DeviceContext* pDevCtx,
  unsigned int index, /* ignored */
  os_boolean write,
  os_uint8* buffer,
  unsigned long length)
{
  CONTROLXRC4LS* pRegs = pDevCtx->hw.control.xrc4ls;
  OS_BAR_HANDLE handle = pDevCtx->hw.controlHandle;
  const os_boolean bByteSwap = FALSE;
  unsigned long i, n8, n32;
  os_uint32 val32;

  if (admxrcDriverParams.model.xrc4lx.smap32Bit) {
    n8 = length & 0x3U;
    n32 = length >> 2;
    if (write) {
      while (n32--) {
	if (bByteSwap) {
	  val32 = (*buffer++ << 24);
	  val32 |= (*buffer++ << 16);
	  val32 |= (*buffer++ << 8);
	  val32 |= (*buffer++ << 0);
	} else {
	  val32 = (*buffer++ << 0);
	  val32 |= (*buffer++ << 8);
	  val32 |= (*buffer++ << 16);
	  val32 |= (*buffer++ << 24);
	}
	os_write_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0], val32);
      }
      if (n8) {
	val32 = 0x0;
	if (bByteSwap) {
	  for (i = 0; i < n8; i++) {
	    val32 = (val32 << 8) | *buffer++;
	  }
	  for (; i < 4; i++) {
	    val32 = (val32 << 8) | 0xffU;
	  }
	} else {
	  for (i = 0; i < n8; i++) {
	    val32 = (val32 >> 8) | (*buffer++ << 24);
	  }
	  for (; i < 4; i++) {
	    val32 = (val32 >> 8) | 0xff000000U;
	  }
	}
	os_write_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0], val32);
      }
      os_read_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0]);
    } else {
      while (n32--) {
	val32 = os_read_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0]);
	*buffer++ = (os_uint8) (val32 >> 0);
	*buffer++ = (os_uint8) (val32 >> 8);
	*buffer++ = (os_uint8) (val32 >> 16);
	*buffer++ = (os_uint8) (val32 >> 24);
      }
      val32 = os_read_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0]);
      while (n8--) {
	*buffer++ = (os_uint8) val32;
	val32 >>= 8;
      }
    }
  } else {
    if (write) {
      while (length--) {
	val32 = *buffer++;
	os_write_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0], val32);
      }
      os_read_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0]);
    } else {
      while (length--) {
	val32 = os_read_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0]);
	*buffer++ = (os_uint8) val32;
      }
      val32 = os_read_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0]);
    }
  }

  return MethodResult_Success;
}

MethodResult
methodSelectMapXrc4sx(
  DeviceContext* pDevCtx,
  unsigned int index, /* ignored */
  os_boolean write,
  os_uint8* buffer,
  unsigned long length)
{
  CONTROLXRC4LS* pRegs = pDevCtx->hw.control.xrc4ls;
  OS_BAR_HANDLE handle = pDevCtx->hw.controlHandle;
  const os_boolean bByteSwap = FALSE;
  unsigned long i, n8, n32;
  os_uint32 val32;

  if (admxrcDriverParams.model.xrc4sx.smap32Bit) {
    n8 = length & 0x3U;
    n32 = length >> 2;
    if (write) {
      while (n32--) {
	if (bByteSwap) {
	  val32 = (*buffer++ << 24);
	  val32 |= (*buffer++ << 16);
	  val32 |= (*buffer++ << 8);
	  val32 |= (*buffer++ << 0);
	} else {
	  val32 = (*buffer++ << 0);
	  val32 |= (*buffer++ << 8);
	  val32 |= (*buffer++ << 16);
	  val32 |= (*buffer++ << 24);
	}
	os_write_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0], val32);
      }
      if (n8) {
	val32 = 0x0;
	if (bByteSwap) {
	  for (i = 0; i < n8; i++) {
	    val32 = (val32 << 8) | *buffer++;
	  }
	  for (; i < 4; i++) {
	    val32 = (val32 << 8) | 0xffU;
	  }
	} else {
	  for (i = 0; i < n8; i++) {
	    val32 = (val32 >> 8) | (*buffer++ << 24);
	  }
	  for (; i < 4; i++) {
	    val32 = (val32 >> 8) | 0xff000000U;
	  }
	}
	os_write_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0], val32);
      }
      os_read_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0]);
    } else {
      while (n32--) {
	val32 = os_read_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0]);
	*buffer++ = (os_uint8)(val32 >> 0);
	*buffer++ = (os_uint8)(val32 >> 8);
	*buffer++ = (os_uint8)(val32 >> 16);
	*buffer++ = (os_uint8)(val32 >> 24);
      }
      val32 = os_read_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0]);
      while (n8--) {
	*buffer++ = (os_uint8) val32;
	val32 >>= 8;
      }
    }
  } else {
    if (write) {
      while (length--) {
	val32 = *buffer++;
	os_write_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0], val32);
      }
      os_read_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0]);
    } else {
      while (length--) {
	val32 = os_read_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0]);
	*buffer++ = (os_uint8) val32;
      }
      val32 = os_read_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0]);
    }
  }
  
  return MethodResult_Success;
}

MethodResult
methodSelectMapXrc4fx(
  DeviceContext* pDevCtx,
  unsigned int index, /* ignored */
  os_boolean write,
  os_uint8* buffer,
  unsigned long length)
{
  XRC4FXSPACE1REGS* pRegs = pDevCtx->hw.control.xrc4fx;
  OS_BAR_HANDLE handle = pDevCtx->hw.controlHandle;
  os_uint8 val8;
	
  if (write) {
    while (length--) {
      val8 = *buffer++;
      os_write_register_le8(handle, &pRegs->FPSMAP[0], val8);
    }
    os_read_register_le32(handle, &pRegs->FPCTL);
  } else {
    while (length--) {
      val8 = os_read_register_le8(handle, &pRegs->FPSMAP[0]);
      *buffer++ = val8;
    }
  }
  
  return MethodResult_Success;
}

MethodResult
methodSelectMapXrce4fx(
  DeviceContext* pDevCtx,
  unsigned int index,
  os_boolean write,
  os_uint8* buffer,
  unsigned long length)
{
  XRCE4FXSPACE1REGS* pRegs = pDevCtx->hw.control.xrce4fx;
  OS_BAR_HANDLE handle = pDevCtx->hw.controlHandle;
  unsigned long n8, n32;
  os_uint32 val32;
  
  n8 = length & 0x3U;
  n32 = length >> 2;
  if (write) {
    while (n32--) {
      val32 = (*buffer++ << 0);
      val32 |= (*buffer++ << 8);
      val32 |= (*buffer++ << 16);
      val32 |= (*buffer++ << 24);
      os_write_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0], val32);
    }
    while (n8--) {
      os_write_register_le8(handle, &pRegs->FPSMAP[0], *buffer++);
    }
  } else {
    while (n32--) {
      val32 = os_read_register_le32(handle, (os_uint32*)&pRegs->FPSMAP[0]);
      *buffer++ = (os_uint8)(val32 >> 0);
      *buffer++ = (os_uint8)(val32 >> 8);
      *buffer++ = (os_uint8)(val32 >> 16);
      *buffer++ = (os_uint8)(val32 >> 24);
    }
    while (n8--) {
      *buffer++ = os_read_register_le8(handle, &pRegs->FPSMAP[0]);
    }
  }
  
  return MethodResult_Success;
}

MethodResult
methodSelectMapBbp(
  DeviceContext* pDevCtx,
  unsigned int index,
  os_boolean write,
  os_uint8* buffer,
  unsigned long length)
{
  BBPSPACE1REGS* pRegs = pDevCtx->hw.control.bbp;
  OS_BAR_HANDLE handle = pDevCtx->hw.controlHandle;
  os_uint8 val8;
  os_uint8* pFpsmap;
  os_uint32* pFpctl;

  if (index == 0) {
    pFpsmap = &pRegs->FPSMAP0[0];
    pFpctl = &pRegs->FPCTL0;
  } else {
    pFpsmap = &pRegs->FPSMAP1[0];
    pFpctl = &pRegs->FPCTL1;
  }
  if (write) {
    while (length--) {
      val8 = *buffer++;
      os_write_register_le8(handle, pFpsmap, val8);
    }
    os_read_register_le32(handle, pFpctl);
  } else {
    while (length--) {
      val8 = os_read_register_le8(handle, pFpsmap);
      *buffer++ = val8;
    }
  }
  
  return MethodResult_Success;
}

