/*
** ADM-XRC Series Driver
**
** (C) Copyright 2001-2006 Alpha Data
**
** methods_info.c - Model-specific routines to read the VPD for a hardware
**                  instance and store it in the device context.
*/

#include "device.h"
#include "drvparam.h"
#include "flash.h"
#include "xplbridge.h"

static void
getSpaceInfoPci9080(
  DeviceContext* pDevCtx)
{
  PLX9080REGS* pRegs = pDevCtx->hw.adapter.pci9080;
  OS_BAR_HANDLE handle = pDevCtx->hw.adapterHandle;
  os_uint32 data32;

  data32 = os_read_register_le32(handle, &pRegs->LAS0BA);
  if (!(data32 & 0x1U)) {
    /* Space not enabled - we don't expect this */
    pDevCtx->info.spaceInfo[0].localBase = 0x0U;
    pDevCtx->info.spaceInfo[0].localSize = 0x0U;
  } else {
    pDevCtx->info.spaceInfo[0].localBase = data32 & ~0xf;
    data32 = os_read_register_le32(handle, &pRegs->LAS0RR);
    pDevCtx->info.spaceInfo[0].localSize = (~(data32 & ~0xf)) + 1;
  }
  pDevCtx->info.spaceInfo[0].physicalBase = (unsigned long) pDevCtx->resources.raw.pci.base[pDevCtx->miscInfo.numBar - 2];
  
  data32 = os_read_register_le32(handle, &pRegs->LAS1BA);
  if (!(data32 & 0x1)) {
    /* Space not enabled - we don't expect this */
    pDevCtx->info.spaceInfo[1].localBase = 0x0U;
    pDevCtx->info.spaceInfo[1].localSize = 0x0U;
  } else {
    pDevCtx->info.spaceInfo[1].localBase = data32 & ~0xf;
    data32 = os_read_register_le32(handle, &pRegs->LAS1RR);
    pDevCtx->info.spaceInfo[1].localSize = (~(data32 & ~0xf)) + 1;
  }
  pDevCtx->info.spaceInfo[1].physicalBase = (unsigned long) pDevCtx->resources.raw.pci.base[pDevCtx->miscInfo.numBar - 1];
  
  pDevCtx->info.cardInfo.numSpace = 2;
}

static void
getSpaceInfoPci9656(
  DeviceContext* pDevCtx)
{
  PLX9656REGS* pRegs = pDevCtx->hw.adapter.pci9656;
  OS_BAR_HANDLE handle = pDevCtx->hw.adapterHandle;
  os_uint32 data32;

  data32 = os_read_register_le32(handle, &pRegs->LAS0BA);
  if (!(data32 & 0x1U)) {
    /* Space not enabled - we don't expect this */
    pDevCtx->info.spaceInfo[0].localBase = 0x0U;
    pDevCtx->info.spaceInfo[0].localSize = 0x0U;
  } else {
    pDevCtx->info.spaceInfo[0].localBase = data32 & ~0xf;
    data32 = os_read_register_le32(handle, &pRegs->LAS0RR);
    pDevCtx->info.spaceInfo[0].localSize = (~(data32 & ~0xf)) + 1;
  }
  pDevCtx->info.spaceInfo[0].physicalBase = (unsigned long) pDevCtx->resources.raw.pci.base[pDevCtx->miscInfo.numBar - 2];
  
  data32 = os_read_register_le32(handle, &pRegs->LAS1BA);
  if (!(data32 & 0x1)) {
    /* Space not enabled - we don't expect this */
    pDevCtx->info.spaceInfo[1].localBase = 0x0U;
    pDevCtx->info.spaceInfo[1].localSize = 0x0U;
  } else {
    pDevCtx->info.spaceInfo[1].localBase = data32 & ~0xf;
    data32 = os_read_register_le32(handle, &pRegs->LAS1RR);
    pDevCtx->info.spaceInfo[1].localSize = (~(data32 & ~0xf)) + 1;
  }
  pDevCtx->info.spaceInfo[1].physicalBase = (unsigned long) pDevCtx->resources.raw.pci.base[pDevCtx->miscInfo.numBar - 1];
  
  if (admxrcDriverParams.pci9656RegsAsSpace) {
    pDevCtx->info.cardInfo.numSpace = 3;
    pDevCtx->info.spaceInfo[2].physicalBase = pDevCtx->resources.raw.pci.base[0];
    pDevCtx->info.spaceInfo[2].localBase = 0;
    pDevCtx->info.spaceInfo[2].localSize = pDevCtx->resources.raw.pci.size[0];
  } else {
    pDevCtx->info.cardInfo.numSpace = 2;
  }
}

static void
getSpaceInfoAdb1(
  DeviceContext* pDevCtx)
{
  XPLBRIDGEREGS* pRegs = pDevCtx->hw.adapter.xpl;
  OS_BAR_HANDLE handle = pDevCtx->hw.adapterHandle;
  os_uint32 val32;

  pDevCtx->info.spaceInfo[0].physicalBase = (unsigned long) pDevCtx->resources.raw.pci.base[pDevCtx->miscInfo.numBar - 2];
  val32 = os_read_register_le32(handle, &pRegs->LS0B) & 0xffff0000;
  pDevCtx->info.spaceInfo[0].localBase = val32;
  val32 = os_read_register_le32(handle, &pRegs->LS0R) & 0xffff0000;
  pDevCtx->info.spaceInfo[0].localSize = ~val32 + 1;

  pDevCtx->info.spaceInfo[1].physicalBase = (unsigned long) pDevCtx->resources.raw.pci.base[pDevCtx->miscInfo.numBar - 1];
  val32 = os_read_register_le32(handle, &pRegs->LS1B) & 0xffff0000;
  pDevCtx->info.spaceInfo[1].localBase = val32;
  val32 = os_read_register_le32(handle, &pRegs->LS1R) & 0xffff0000;
  pDevCtx->info.spaceInfo[1].localSize = ~val32 + 1;
}

/*
** Read EEPROM info on an ADM-XRC / ADM-XRC-P / ADM-XRCII-Lite.
**
** There are some differences between the ADM-XRC, ADM-XRC-P
** and the ADM-XRCIILite in the encoding of the EEPROM information,
** but they have enough in common for this routine to handle all 3
** models.
*/
MethodResult
methodInitCardInfoXrc(
  DeviceContext* pDevCtx)
{
  os_uint32 data16, data32;
  unsigned long i;

  os_dbgprint(1, ("methodInitCardInfoPci9080: entered\n"));

  pDevCtx->info.adapterLimit = 0x100;

  switch (pDevCtx->devId.pci.subDevice) {
  case PCI_SUBDEVICE_ADATA_ADMXRC:
    /*
    ** ADM-XRC or ADM-XRC-P
    **
    ** Location 0x36: board type
    */
    pDevCtx->methods.configMem(pDevCtx, FALSE, 0x36, &data16);
    strcpy(pDevCtx->info.fpgaInfo[0].package, "BG560");
    if (data16 == ADMXRC2_BOARD_ADMXRC) {
      pDevCtx->info.cardInfo.boardType = (ADMXRC2_BOARD_TYPE) data16;
      pDevCtx->clock.lclkMin = admxrcDriverParams.model.xrc.lclkMin;
      pDevCtx->clock.lclkMax = admxrcDriverParams.model.xrc.lclkMax;
    } else if (data16 == ADMXRC2_BOARD_ADMXRC_P) {
      pDevCtx->info.cardInfo.boardType = (ADMXRC2_BOARD_TYPE) data16;
      pDevCtx->clock.lclkMin = admxrcDriverParams.model.xrcp.lclkMin;
      pDevCtx->clock.lclkMax = admxrcDriverParams.model.xrcp.lclkMax;
    } else {
      /*
      ** Board type must be recorded wrongly in EEPROM,
      ** assume ADM-XRC.
      */
      pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC;
      pDevCtx->clock.lclkMin = admxrcDriverParams.model.xrc.lclkMin;
      pDevCtx->clock.lclkMax = admxrcDriverParams.model.xrc.lclkMax;
    }
    break;

  case PCI_SUBDEVICE_ADATA_ADMXRC2L:
    /*
    ** ADM-XRC2-L
    **
    ** Don't use the board type from the EEPROM, as
    ** the PCI subsystem device ID for the ADM-XRC2-L
    ** is sufficient to correctly identify it.
    */
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC2_LITE;
    pDevCtx->clock.lclkMin = admxrcDriverParams.model.xrc2l.lclkMin;
    pDevCtx->clock.lclkMax = admxrcDriverParams.model.xrc2l.lclkMax;
    strcpy(pDevCtx->info.fpgaInfo[0].package, "FG456");
    break;

  default:
    os_assert(FALSE);
    return MethodResult_GeneralFailure;
  }

  pDevCtx->info.cardInfo.numClock = 2;
  pDevCtx->info.cardInfo.numDmaChan = pDevCtx->dma.info.numChan;

  /*
  ** Location 0x37: modifications word
  **
  ** The modifications word is interpreted in a board
  ** specific manner.
  */
  pDevCtx->info.modifications = 0x0;
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x37, &data16);
  switch (pDevCtx->info.cardInfo.boardType) {
  case ADMXRC2_BOARD_ADMXRC:
  case ADMXRC2_BOARD_ADMXRC_P:
    /*
    ** ADM-XRC/ADM-XRC-P:
    **
    ** Bit     Meaning
    **
    ** 0       1 => clock swap mod has been performed
    */
    if (data16 & 0x1) {
      pDevCtx->info.modifications |= ADMXRC_MOD_CLOCKSWAP;
    }
    break;

  case ADMXRC2_BOARD_ADMXRC2_LITE:
    /*
    ** ADM-XRC-II-L: no modifications currently defined
    */
    break;

  default:
    os_assert(FALSE);
    return MethodResult_GeneralFailure;
  }
  
  /* Location 0x38: clock oscillator type (ie. frequency) */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x38, &data16);  
  pDevCtx->info.clockType = (ClockType) data16;
  
  /* Location 0x39: flash type (ie. device) - IGNORED */
  
  /* Location 0x3B: card ID */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3B, &data16);
  pDevCtx->info.cardInfo.cardId = (ADMXRC2_CARDID) data16;
  
  /* Location 0x3C: fpga type (ie. device) */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3C, &data16);
  pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE)data16;
  pDevCtx->info.fpgaInfo[0].type = (ADMXRC2_FPGA_TYPE)data16;
  
  /* Location 0x3D: serial number low 16 bits */
  /* Location 0x3E: serial number high 16 bits */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3D, &data16);
  data32 = data16;
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3E, &data16);
  data32 |= (data16 << 16);
  pDevCtx->info.cardInfo.serialNum = data32;
  
  /*
  ** Location 0x3F:
  **    low 8 bits = logic rev
  **    high 8 bits = board rev
  */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3F, &data16);
  pDevCtx->info.cardInfo.boardRevision = (os_uint8) ((data16 & 0xff00U) >> 8);
  pDevCtx->info.cardInfo.logicRevision = (os_uint8) ((data16 & 0x00ffU) >> 0);
  
  /* Location 0x3A: SRAM presence info */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3A, &data16);
  switch (pDevCtx->info.cardInfo.boardType) {
  case ADMXRC2_BOARD_ADMXRC:
  case ADMXRC2_BOARD_ADMXRC_P:
    {
      /*
      ** ADM-XRC/ADM-XRC-P: get bank size/presence info, 4 banks
      **
      ** 4 banks of x36 SSRAM
      */
      os_boolean found = FALSE;
      
      pDevCtx->info.cardInfo.numRamBank = 4;
      if ((data16 & 0xf) == 0xfU) {
	pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
	for (i = 0; i < 4; i++) {
	  pDevCtx->info.bankInfo[i].fitted = TRUE;
	  pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_ZBTFT;
	  pDevCtx->info.bankInfo[i].width = 36;
	  if (data16 & (1 << (i + 4))) {
	    /* 16Mbit */
	    pDevCtx->info.bankInfo[i].size = 512*1024;
	  } else {
	    /* 8Mbit */
	    pDevCtx->info.bankInfo[i].size = 256*1024;
	  }
	  pDevCtx->info.cardInfo.ramBanksFitted |= (1 << i);
	}
	found = TRUE;
      }
      if (!found) {
	for (i = 0; i < 4; i++) {
	  pDevCtx->info.bankInfo[i].fitted = FALSE;
	  pDevCtx->info.bankInfo[i].type = 0;
	  pDevCtx->info.bankInfo[i].width = 36;
	  pDevCtx->info.bankInfo[i].size = 0;
	}
	pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
      }
      break;
    }

  case ADMXRC2_BOARD_ADMXRC2_LITE:
    {
      /*	
      ** ADM-XRC-IIL: get bank size/presence info
      **
      ** 4 banks of x18 SSRAM
      */
      os_uint16  bank;
      os_boolean fitted = FALSE; /* Shut the compiler up */
      
      pDevCtx->info.cardInfo.numRamBank = 4;
      for (i = 0; i < pDevCtx->info.cardInfo.numRamBank; i++) {
	bank = (data16 >> (i << 2)) & 0xf;
	switch (bank & 0x3) {
	case 0x0:
	  /* not fitted */
	  fitted = FALSE;
	  pDevCtx->info.bankInfo[i].type = 0;
	  break;
	case 0x1:
	  /* ZBT flow-through */
	  fitted = TRUE;
	  pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_ZBTFT;
	  break;
	case 0x2:
	  /* ZBT pipelined */
	  fitted = TRUE;
	  pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_ZBTP;
	  break;
	case 0x3:
	  /* ZBT flow-through + pipelined (selectable) */
	  fitted = TRUE;
	  pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_ZBTFT | ADMXRC2_RAM_ZBTP;
	  break;
	}
	pDevCtx->info.bankInfo[i].width = 18;
	if (!fitted) {
	  pDevCtx->info.bankInfo[i].size = 0;
	} else {
	  switch ((bank >> 2) & 0x3) {
	  case 0x0:
	    /* 8Mbit */
	    pDevCtx->info.bankInfo[i].size = 512*1024;
	    break;
	  case 0x1:
	    /* 16Mbit */
	    pDevCtx->info.bankInfo[i].size = 1024*1024;
	    break;
	  case 0x2:
	    /* 32Mbit */
	    pDevCtx->info.bankInfo[i].size = 2048*1024;
	    break;
	  case 0x3:
	    /* reserved */
	    pDevCtx->info.bankInfo[i].size = 0;
	    fitted = FALSE;
	    break;
	  }
	}
	pDevCtx->info.bankInfo[i].fitted = fitted;
	if (fitted) {
	  pDevCtx->info.cardInfo.ramBanksFitted |= (1 << i);
	}
      }
      break;
    }

  default:
    os_assert(FALSE);
    return MethodResult_GeneralFailure;
  }

  getSpaceInfoPci9080(pDevCtx);

  return MethodResult_Success;
}

/*
** Read EEPROM info on an ADM-XRCII (PCI9656 bridge).
*/
MethodResult
methodInitCardInfoXrc2(
  DeviceContext* pDevCtx)
{
  os_uint32     data16, data32;
  unsigned long i;

  os_dbgprint(1, ("methodInitCardInfoXrc2: entered\n"));

  pDevCtx->info.adapterLimit = 0x100;

  pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC2;
  pDevCtx->info.cardInfo.numClock = 2;
  pDevCtx->info.cardInfo.numDmaChan = pDevCtx->dma.info.numChan;
  pDevCtx->clock.lclkMin = admxrcDriverParams.model.xrc2.lclkMin;
  pDevCtx->clock.lclkMax = admxrcDriverParams.model.xrc2.lclkMax;


  /*
  ** Location 0x36: modifications word
  **
  ** The modifications word is interpreted in a board
  ** specific manner.
  */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x36, &data16);
  pDevCtx->info.modifications = 0x0; /* No modifications defined yet */

  /* Location 0x37: clock oscillator type (ie. frequency) */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x37, &data16);
  pDevCtx->info.clockType = (ClockType) data16;
  
  /* Location 0x38: flash type (ie. device) - IGNORED */

  /* Location 0x3B: card ID */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3B, &data16);
  pDevCtx->info.cardInfo.cardId = (ADMXRC2_CARDID) data16;
  
  /* Location 0x3C: fpga type (ie. device) */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3C, &data16);
  pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE)data16;
  pDevCtx->info.fpgaInfo[0].type = (ADMXRC2_FPGA_TYPE)data16;
  strcpy(pDevCtx->info.fpgaInfo[0].package, "FF1152");

  /* Location 0x3D: serial number low 16 bits */
  /* Location 0x3E: serial number high 16 bits */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3D, &data16);
  data32 = data16;
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3E, &data16);
  data32 |= (data16 << 16);
  pDevCtx->info.cardInfo.serialNum = data32;

  /*
  ** Location 0x3F:
  **    low 8 bits = logic rev
  **    high 8 bits = board rev
  */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3F, &data16);
  pDevCtx->info.cardInfo.boardRevision = (os_uint8) ((data16 & 0xff00) >> 8);
  pDevCtx->info.cardInfo.logicRevision = (os_uint8) ((data16 & 0x00ff) >> 0);
  
  /* Locations 0x39 & 0x3A: SRAM presence info */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x39, &data16);
  data32 = data16;
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3A, &data16);
  data32 |= data16 << 16U;
  {
    /*
    ** ADM-XRC2: get bank size/presence info, 6 banks
    */
    unsigned long bank;
    os_boolean fitted = FALSE;
    
    pDevCtx->info.cardInfo.numRamBank = 6;
    pDevCtx->info.cardInfo.ramBanksFitted = 0;
    for (i = 0; i < pDevCtx->info.cardInfo.numRamBank; i++) {
      bank = (data32 >> (i << 2)) & 0xf;
      switch (bank & 0x3) {
      case 0x0:
	/* not fitted */
	fitted = FALSE;
	pDevCtx->info.bankInfo[i].type = 0;
	break;

      case 0x1:
	/* ZBT flow-through */
	fitted = TRUE;
	pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_ZBTFT;
	break;

      case 0x2:
	/* ZBT pipelined */
	fitted = TRUE;
	pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_ZBTP;
	break;

      case 0x3:
	/* ZBT flow-through + pipelined (selectable) */
	fitted = TRUE;
	pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_ZBTFT | ADMXRC2_RAM_ZBTP;
	break;

      default:
	os_assert(FALSE);
	break;
      }
      pDevCtx->info.bankInfo[i].width = (pDevCtx->info.cardInfo.fpgaType == ADMXRC2_FPGA_2V3000) ? 32 : 36;
      if (!fitted) {
	pDevCtx->info.bankInfo[i].size = 0;
      } else {
	switch ((bank >> 2) & 0x3) {
	case 0x0:
	  /* 8Mbit */
	  pDevCtx->info.bankInfo[i].size = 256*1024;
	  break;

	case 0x1:
	  /* 16Mbit */
	  pDevCtx->info.bankInfo[i].size = 512*1024;
	  break;

	case 0x2:
	  /* 32Mbit */
	  pDevCtx->info.bankInfo[i].size = 1024*1024;
	  break;

	case 0x3:
	  /* 64Mbit */
	  pDevCtx->info.bankInfo[i].size = 2048*1024;
	  fitted = FALSE;
	  break;

	default:
	  os_assert(FALSE);
	  break;
	}
      }
      pDevCtx->info.bankInfo[i].fitted = fitted;
      if (fitted) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (1 << i);
      }
    }
  }

  getSpaceInfoPci9656(pDevCtx);

  return MethodResult_Success;
}

/*
** Read configuration information on an ADM-XPL (actually read
** from a Flash memory attached to the PCI bridge).
*/
MethodResult
methodInitCardInfoXpl(
  DeviceContext* pDevCtx)
{
  XPLCONFIGREV0	 config;
  unsigned int	 i;
  os_boolean     bConfigFound = FALSE;
  
  os_dbgprint(3, ("methodInitCardInfoXpl: entered\n"));
  
  pDevCtx->info.adapterLimit = 0x100;
  pDevCtx->clock.lclkMin = admxrcDriverParams.model.xpl.lclkMin;
  pDevCtx->clock.lclkMax = admxrcDriverParams.model.xpl.lclkMax;
  
  if (!admxrcDriverParams.model.xpl.unlockFlash) {
    os_uint16 version;

    if (readFlash(pDevCtx, 0, sizeof(config), &config) == MethodResult_Success) {
      version = os_le16_to_cpu(config.version);
      switch (version) {
      case 0:
	bConfigFound = TRUE;
	break;
	
      default:
	/* Unrecognised data structure */
	break;
      }
    }
  }

  if (!bConfigFound) {
    os_dbgprint(0, ("*** methodInitCardInfoXpl: no configuration information found\n"));
  }

  if (bConfigFound) {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXPL;
    pDevCtx->info.cardInfo.numClock = 1;
    pDevCtx->info.cardInfo.cardId = os_le32_to_cpu(config.cardId);
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE)os_le16_to_cpu(config.fpgaType);
    pDevCtx->info.cardInfo.serialNum = os_le32_to_cpu(config.serialNumber);
    pDevCtx->info.cardInfo.boardRevision = os_le8_to_cpu(config.pcbRev);
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numDmaChan = pDevCtx->dma.info.numChan;
    pDevCtx->info.cardInfo.numRamBank = 2;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = 0x0;
    pDevCtx->info.clockType = ClockType_14;
    for (i = 0; i < 2; i++) {
      os_uint16 typeMask;
      os_uint8 width, order;

      typeMask = os_le16_to_cpu(config.bank[i].typeMask);
      width = os_le8_to_cpu(config.bank[i].width);
      order = os_le8_to_cpu(config.bank[i].order);
      pDevCtx->info.bankInfo[i].type = typeMask;
      pDevCtx->info.bankInfo[i].width = width;
      if (order && typeMask && width) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << i);
	pDevCtx->info.bankInfo[i].fitted = TRUE;
	pDevCtx->info.bankInfo[i].size = 1 << order;
      } else {
	pDevCtx->info.bankInfo[i].fitted = FALSE;
	pDevCtx->info.bankInfo[i].size = 0;
      }
    }
  } else {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXPL;
    pDevCtx->info.cardInfo.numClock = 1;
    pDevCtx->info.cardInfo.numDmaChan = 0;
    pDevCtx->info.cardInfo.cardId = 0xdeadU;
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE) ADMXRC2_FPGA_2VP7;
    pDevCtx->info.cardInfo.serialNum = 0xcafefaceU;
    pDevCtx->info.cardInfo.boardRevision = 0x10;
    pDevCtx->info.cardInfo.logicRevision = 0x10;
    pDevCtx->info.cardInfo.numRamBank = 2;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = 0x0;
    pDevCtx->info.clockType = ClockType_14;
    pDevCtx->info.bankInfo[0].fitted = FALSE;
    pDevCtx->info.bankInfo[0].type = ADMXRC2_RAM_ZBTP;
    pDevCtx->info.bankInfo[0].size = 0;
    pDevCtx->info.bankInfo[0].width = 64;
    pDevCtx->info.bankInfo[1].fitted = FALSE;
    pDevCtx->info.bankInfo[1].type = ADMXRC2_RAM_SDRAM_DDR;
    pDevCtx->info.bankInfo[1].size = 0;
    pDevCtx->info.bankInfo[1].width = 64;
  }
  strcpy(pDevCtx->info.fpgaInfo[0].package, "FF896");
  pDevCtx->info.fpgaInfo[0].type = pDevCtx->info.cardInfo.fpgaType;
  
  getSpaceInfoAdb1(pDevCtx);

  return bConfigFound ? MethodResult_Success : MethodResult_NotPresent;
}

/*
** Read EEPROM info on an ADM-XRCII-N (PCI9656 bridge).
*/
MethodResult
methodInitCardInfoWrc2(
  DeviceContext* pDevCtx)
{
  os_uint32     data16, data32;
  unsigned long i;

  os_dbgprint(1, ("methodInitCardInfoWrc2: entered\n"));

  pDevCtx->info.adapterLimit = 0x100;
  pDevCtx->clock.lclkMin = admxrcDriverParams.model.wrc2.lclkMin;
  pDevCtx->clock.lclkMax = admxrcDriverParams.model.wrc2.lclkMax;
  pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADPWRC2;
  pDevCtx->info.cardInfo.numClock = 2;
  pDevCtx->info.clockType = 0x0001U; /* 14.318MHz */
  pDevCtx->info.cardInfo.numDmaChan = pDevCtx->dma.info.numChan;
  pDevCtx->info.cardInfo.numRamBank = 2;
  pDevCtx->info.cardInfo.ramBanksFitted = 0;

  /*
  ** Location 0x36: modifications word
  **
  ** The modifications word is interpreted in a board
  ** specific manner.
  */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x36, &data16);
  pDevCtx->info.modifications = 0x0; /* No modifications defined yet */

  /* Location 0x37: clock oscillator type (ie. frequency) */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x37, &data16);
  pDevCtx->info.clockType = (ClockType) data16;
  
  /* Location 0x38: flash type (ie. device) - IGNORED */

  /* Location 0x3B: card ID */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3B, &data16);
  pDevCtx->info.cardInfo.cardId = (ADMXRC2_CARDID) data16;
  
  /* Location 0x3C: fpga type (ie. device) */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3C, &data16);
  pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE)data16;
  strcpy(pDevCtx->info.fpgaInfo[0].package, "FF1517");
  pDevCtx->info.fpgaInfo[0].type = pDevCtx->info.cardInfo.fpgaType;

  /* Location 0x3D: serial number low 16 bits */
  /* Location 0x3E: serial number high 16 bits */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3D, &data16);
  data32 = data16;
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3E, &data16);
  data32 |= (data16 << 16);
  pDevCtx->info.cardInfo.serialNum = data32;

  /*
  ** Location 0x3F:
  **    low 8 bits = logic rev
  **    high 8 bits = board rev
  */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3F, &data16);
  pDevCtx->info.cardInfo.boardRevision = (os_uint8) ((data16 & 0xff00) >> 8);
  pDevCtx->info.cardInfo.logicRevision = (os_uint8) ((data16 & 0x00ff) >> 0);
  
  /* Locations 0x39 & 0x3A: SRAM presence info */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x39, &data16);
  data32 = data16;
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3A, &data16);
  data32 |= data16 << 16U;

  /* Locations 0x39-0x3A: memory bank presence info */
  for (i = 0; i < 2; i++) {
    pDevCtx->methods.configMem(pDevCtx, FALSE, 0x39U + i, &data32);
    if (!(data32 & 0xffU)) {
      /* Bank not fitted */
      pDevCtx->info.bankInfo[i].fitted = FALSE;
      pDevCtx->info.bankInfo[i].size = 0;
    } else {
      pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << i);
      pDevCtx->info.bankInfo[i].fitted = TRUE;
      pDevCtx->info.bankInfo[i].size = 0x1U << (data32 & 0xffU);
    }
    
    pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_SDRAM_DDR;
    pDevCtx->info.bankInfo[i].width = 512;
  }

  getSpaceInfoPci9656(pDevCtx);

  return MethodResult_Success;
}

/*
** Read EEPROM info on an ADM-XRCII-EMP (PCI9656 bridge).
*/
MethodResult
methodInitCardInfoDrc2(
  DeviceContext* pDevCtx)
{
  os_uint32     data16, data32;
  unsigned long i;

  os_dbgprint(1, ("methodInitCardInfoDrc2: entered\n"));

  pDevCtx->info.adapterLimit = 0x100;
  pDevCtx->clock.lclkMin = admxrcDriverParams.model.drc2.lclkMin;
  pDevCtx->clock.lclkMax = admxrcDriverParams.model.drc2.lclkMax;
  pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADPDRC2;
  pDevCtx->info.cardInfo.numClock = 2;
  pDevCtx->info.cardInfo.numDmaChan = pDevCtx->dma.info.numChan;
  pDevCtx->info.clockType = 0x0001U; /* 14.318MHz */
  pDevCtx->info.cardInfo.numRamBank = 5;
  pDevCtx->info.cardInfo.ramBanksFitted = 0;

  /*
  ** Location 0x35: modifications word
  **
  ** The modifications word is interpreted in a board
  ** specific manner.
  */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x35, &data16);
  pDevCtx->info.modifications = 0x0; /* No modifications defined yet */

  /* Location 0x3B: card ID */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3B, &data16);
  pDevCtx->info.cardInfo.cardId = (ADMXRC2_CARDID) data16;
  
  /* Location 0x3C: fpga type (ie. device) */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3C, &data16);
  pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE)data16;
  strcpy(pDevCtx->info.fpgaInfo[0].package, "FF1152");
  pDevCtx->info.fpgaInfo[0].type = pDevCtx->info.cardInfo.fpgaType;

  /* Location 0x3D: serial number low 16 bits */
  /* Location 0x3E: serial number high 16 bits */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3D, &data16);
  data32 = data16;
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3E, &data16);
  data32 |= (data16 << 16);
  pDevCtx->info.cardInfo.serialNum = data32;

  /*
  ** Location 0x3F:
  **    low 8 bits = logic rev
  **    high 8 bits = board rev
  */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3F, &data16);
  pDevCtx->info.cardInfo.boardRevision = (os_uint8) ((data16 & 0xff00) >> 8);
  pDevCtx->info.cardInfo.logicRevision = (os_uint8) ((data16 & 0x00ff) >> 0);
  
  /* Locations 0x39 & 0x3A: SRAM presence info */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x39, &data16);
  data32 = data16;
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3A, &data16);
  data32 |= data16 << 16U;

  /* Locations 0x36-0x3A: memory bank presence info */
  for (i = 0; i < 4; i++) {
    os_uint32 type;
    
    pDevCtx->methods.configMem(pDevCtx, FALSE, 0x36U + i, &data32);
    if (!(data32 & 0xffU)) {
      /* Bank not fitted */
      pDevCtx->info.bankInfo[i].fitted = FALSE;
      pDevCtx->info.bankInfo[i].size = 0;
    } else {
      pDevCtx->info.bankInfo[i].fitted = TRUE;
      pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << i);
      pDevCtx->info.bankInfo[i].size = 0x1U << (data32 & 0xffU);
    }
    type = (data32 & 0x7f00U) >> 8;
    pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_SDRAM_DDR;
    pDevCtx->info.bankInfo[i].width = 128;
  }
  
  for (i = 4; i < 5; i++) {
    pDevCtx->methods.configMem(pDevCtx, FALSE, 0x36U + i, &data32);
    if (!(data32 & 0xffU)) {
      /* Bank not fitted */
      pDevCtx->info.bankInfo[i].fitted = FALSE;
      pDevCtx->info.bankInfo[i].size = 0;
    } else {
      pDevCtx->info.bankInfo[i].fitted = TRUE;
      pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << i);
      pDevCtx->info.bankInfo[i].size = 0x1U << (data32 & 0xffU);
    }
    if (data32 & 0x8000U) {
      pDevCtx->info.bankInfo[i].width = 72;
    } else {
      pDevCtx->info.bankInfo[i].width = 64;
    }
    pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_SRAM_DDR2;
  }
  
  getSpaceInfoPci9656(pDevCtx);

  return MethodResult_Success;
}

/*
** Read configuration information on an ADM-XP (actually read
** from a Flash memory attached to the PCI bridge).
*/
MethodResult
methodInitCardInfoXp(
  DeviceContext* pDevCtx)
{
  XPCONFIGREV0	 config;
  unsigned int	 i;
  os_boolean     bConfigFound = FALSE;
  
  os_dbgprint(3, ("methodInitCardInfoXp: entered\n"));
  
  pDevCtx->info.adapterLimit = 0x100;
  pDevCtx->clock.lclkMin = admxrcDriverParams.model.xp.lclkMin;
  pDevCtx->clock.lclkMax = admxrcDriverParams.model.xp.lclkMax;
  
  if (!admxrcDriverParams.model.xp.unlockFlash) {
    os_uint16 version;

    if (readFlash(pDevCtx, 0, sizeof(config), &config) == MethodResult_Success) {
      version = os_le16_to_cpu(config.version);
      switch (version) {
      case 0:
	bConfigFound = TRUE;
	break;
	
      default:
	/* Unrecognised data structure */
	break;
      }
    }
  }

  if (!bConfigFound) {
    os_dbgprint(0, ("*** methodInitCardInfoXp: no configuration information found\n"));
  }

  if (bConfigFound) {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC2_PRO;
    pDevCtx->info.cardInfo.numClock = 1;
    pDevCtx->info.cardInfo.cardId = os_le32_to_cpu(config.cardId);
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE) os_le16_to_cpu(config.fpgaType);
    pDevCtx->info.cardInfo.serialNum = os_le32_to_cpu(config.serialNumber);
    pDevCtx->info.cardInfo.boardRevision = os_le8_to_cpu(config.pcbRev);
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numDmaChan = pDevCtx->dma.info.numChan;
    pDevCtx->info.cardInfo.numRamBank = 6;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = 0x0;
    pDevCtx->info.clockType = ClockType_14;
    for (i = 0; i < 4; i++) {
      os_uint8 order;

      pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_SRAM_DDR2;
      pDevCtx->info.bankInfo[i].width = 64;
      order = os_le8_to_cpu(config.zbt[i].order);
      if (order) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << i);
	pDevCtx->info.bankInfo[i].fitted = TRUE;
	pDevCtx->info.bankInfo[i].size = 1 << order;
      } else {
	pDevCtx->info.bankInfo[i].fitted = FALSE;
	pDevCtx->info.bankInfo[i].size = 0;
      }
    }
    for (i = 0; i < 2; i++) {
      os_uint8 order;

      pDevCtx->info.bankInfo[i + 4].type = ADMXRC2_RAM_SDRAM_DDR;
      pDevCtx->info.bankInfo[i + 4].width = 64;
      order = os_le8_to_cpu(config.sdram[i].order);
      if (order) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << (i + 4));
	pDevCtx->info.bankInfo[i + 4].fitted = TRUE;
	pDevCtx->info.bankInfo[i + 4].size = 1 << order;
      } else {
	pDevCtx->info.bankInfo[i + 4].fitted = FALSE;
	pDevCtx->info.bankInfo[i + 4].size = 0;
      }
    }
  } else {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC2_PRO;
    pDevCtx->info.cardInfo.numClock = 1;
    pDevCtx->info.cardInfo.numDmaChan = 2;
    pDevCtx->info.cardInfo.cardId = 0xdeadU;
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE) ADMXRC2_FPGA_2VP70;
    pDevCtx->info.cardInfo.serialNum = 0xcafefaceU;
    pDevCtx->info.cardInfo.boardRevision = 0x10;
    pDevCtx->info.cardInfo.logicRevision = 0x10;
    pDevCtx->info.cardInfo.numRamBank = 6;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = 0x0;
    pDevCtx->info.clockType = ClockType_14;
    for (i = 0; i < 4; i++) {
      pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_SRAM_DDR2;
      pDevCtx->info.bankInfo[i].width = 64;
      pDevCtx->info.bankInfo[i].fitted = FALSE;
      pDevCtx->info.bankInfo[i].size = 0;
    }
    for (i = 0; i < 2; i++) {
      pDevCtx->info.bankInfo[i + 4].type = ADMXRC2_RAM_SDRAM_DDR;
      pDevCtx->info.bankInfo[i + 4].width = 64;
      pDevCtx->info.bankInfo[i + 4].fitted = FALSE;
      pDevCtx->info.bankInfo[i + 4].size = 0;
    }
  }
  strcpy(pDevCtx->info.fpgaInfo[0].package, "FF1704");
  pDevCtx->info.fpgaInfo[0].type = pDevCtx->info.cardInfo.fpgaType;
  
  getSpaceInfoAdb1(pDevCtx);

  return bConfigFound ? MethodResult_Success : MethodResult_NotPresent;
}

/*
** Read configuration information on an ADP-XPI (actually read
** from a Flash memory attached to the PCI bridge).
*/
MethodResult
methodInitCardInfoXpi(
  DeviceContext* pDevCtx)
{
  XPICONFIGREV0	 config;
  unsigned int	 i;
  os_boolean     bConfigFound = FALSE;
  
  os_dbgprint(3, ("methodInitCardInfoXpi: entered\n"));
  
  pDevCtx->info.adapterLimit = 0x100;
  pDevCtx->clock.lclkMin = admxrcDriverParams.model.xpi.lclkMin;
  pDevCtx->clock.lclkMax = admxrcDriverParams.model.xpi.lclkMax;
  
  if (!admxrcDriverParams.model.xpi.unlockFlash) {
    if (readFlash(pDevCtx, 0, sizeof(config), &config) == MethodResult_Success) {
      os_uint16 version;

      version = os_le16_to_cpu(config.version);
      switch (version) {
      case 0:
	bConfigFound = TRUE;
	break;
	
      default:
	/* Unrecognised data structure */
	break;
      }
    }
  }

  if (!bConfigFound) {
    os_dbgprint(0, ("*** methodInitCardInfoXpi: no configuration information found\n"));
  }

  if (bConfigFound) {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADPXPI;
    pDevCtx->info.cardInfo.numClock = 1;
    pDevCtx->info.cardInfo.cardId = os_le32_to_cpu(config.cardId);
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE) os_le16_to_cpu(config.fpgaType);
    pDevCtx->info.cardInfo.serialNum = os_le32_to_cpu(config.serialNumber);
    pDevCtx->info.cardInfo.boardRevision = os_le8_to_cpu(config.pcbRev);
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numDmaChan = pDevCtx->dma.info.numChan;
    pDevCtx->info.cardInfo.numRamBank = 5;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = 0x0;
    pDevCtx->info.clockType = ClockType_14;
    for (i = 0; i < 4; i++) {
      os_uint8 order;

      pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_SDRAM_DDR;
      pDevCtx->info.bankInfo[i].width = 128;
      order = os_le8_to_cpu(config.sdram[i].order);
      if (order) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << i);
	pDevCtx->info.bankInfo[i].fitted = TRUE;
	pDevCtx->info.bankInfo[i].size = 1 << order;
      } else {
	pDevCtx->info.bankInfo[i].fitted = FALSE;
	pDevCtx->info.bankInfo[i].size = 0;
      }
    }
    for (i = 0; i < 1; i++) {
      os_uint8 order;

      pDevCtx->info.bankInfo[i + 4].type = ADMXRC2_RAM_SRAM_DDR2;
      pDevCtx->info.bankInfo[i + 4].width = 64;
      order = config.sram[i].order;
      if (order) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << (i + 4));
	pDevCtx->info.bankInfo[i + 4].fitted = TRUE;
	pDevCtx->info.bankInfo[i + 4].size = 1 << order;
      } else {
	pDevCtx->info.bankInfo[i + 4].fitted = FALSE;
	pDevCtx->info.bankInfo[i + 4].size = 0;
      }
    }
  } else {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADPXPI;
    pDevCtx->info.cardInfo.numClock = 1;
    pDevCtx->info.cardInfo.numDmaChan = 2;
    pDevCtx->info.cardInfo.cardId = 0xdeadU;
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE) ADMXRC2_FPGA_2VP70;
    pDevCtx->info.cardInfo.serialNum = 0xcafefaceU;
    pDevCtx->info.cardInfo.boardRevision = 0x10;
    pDevCtx->info.cardInfo.logicRevision = 0x10;
    pDevCtx->info.cardInfo.numRamBank = 6;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = 0x0;
    pDevCtx->info.clockType = ClockType_14;
    for (i = 0; i < 4; i++) {
      pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_SDRAM_DDR;
      pDevCtx->info.bankInfo[i].width = 128;
      pDevCtx->info.bankInfo[i].fitted = FALSE;
      pDevCtx->info.bankInfo[i].size = 0;
    }
    for (i = 0; i < 1; i++) {
      pDevCtx->info.bankInfo[i + 4].type = ADMXRC2_RAM_SRAM_DDR2;
      pDevCtx->info.bankInfo[i + 4].width = 64;
      pDevCtx->info.bankInfo[i + 4].fitted = FALSE;
      pDevCtx->info.bankInfo[i + 4].size = 0;
    }
  }
  strcpy(pDevCtx->info.fpgaInfo[0].package, "FF1704");
  pDevCtx->info.fpgaInfo[0].type = pDevCtx->info.cardInfo.fpgaType;
  
  getSpaceInfoAdb1(pDevCtx);

  return bConfigFound ? MethodResult_Success : MethodResult_NotPresent;
}

MethodResult
methodInitCardInfoXrc4ls(
  DeviceContext* pDevCtx)
{
  os_uint32			data32;
  os_uint32			bankInfo;
  unsigned int	i;
	
  os_dbgprint(3, ("methodInitCardInfoXrc4ls: entered\n"));

  pDevCtx->info.adapterLimit = 0x200;

  pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC4LS;
  pDevCtx->info.cardInfo.numClock = 2;
  pDevCtx->info.cardInfo.numDmaChan = pDevCtx->dma.info.numChan;
  pDevCtx->clock.lclkMin = admxrcDriverParams.model.xrc4ls.lclkMin;
  pDevCtx->clock.lclkMax = admxrcDriverParams.model.xrc4ls.lclkMax;

  /*
  ** Location 0x36: modifications word
  **
  ** The modifications word is interpreted in a board
  ** specific manner.
  */
  pDevCtx->info.modifications = 0x0;
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x36U, &data32);
  
  /* Location 0x37: clock oscillator type (ie. frequency) */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x37U, &data32);
  pDevCtx->info.clockType = (os_uint16) data32;
  
  /* Location 0x38: flash type (ie. device) - IGNORED */
  
  /* Location 0x3B: card ID */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3bU, &data32);
  pDevCtx->info.cardInfo.cardId = (ADMXRC2_CARDID) data32;
  
  /* Location 0x3C: fpga type (ie. device) */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3cU, &data32);
  pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE) data32;
  strcpy(pDevCtx->info.fpgaInfo[0].package, "FF1148");
  pDevCtx->info.fpgaInfo[0].type = pDevCtx->info.cardInfo.fpgaType;
  
  /* Location 0x3D: serial number low 16 bits */
  /* Location 0x3E: serial number high 16 bits */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3dU, &data32);
  pDevCtx->info.cardInfo.serialNum = data32 & 0xffffU;
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3eU, &data32);
  pDevCtx->info.cardInfo.serialNum |= (data32 & 0xffffU) << 16;
  
  /*
  ** Location 0x3F:
  **    low 8 bits = logic rev
  **    high 8 bits = board rev
  */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3fU, &data32);
  pDevCtx->info.cardInfo.boardRevision = (os_uint8) ((data32 >> 8) & 0xffU);
  pDevCtx->info.cardInfo.logicRevision = (os_uint8) ((data32 >> 0) & 0xffU);
  
  /* Locations 0x39 & 0x3A: SRAM presence info */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x39U, &data32);
  bankInfo = data32 & 0xffffU;
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3aU, &data32);
  bankInfo |= (data32 & 0xffff) << 16;
  
  {
    /*
    ** ADM-XRC4LS: get bank size/presence info, 6 banks
    */
    os_uint32 bank;
    os_boolean fitted = FALSE;
    
    pDevCtx->info.cardInfo.numRamBank = 6;
    pDevCtx->info.cardInfo.ramBanksFitted = 0;
    for (i = 0; i < pDevCtx->info.cardInfo.numRamBank; i++) {
      bank = (bankInfo >> (i << 2)) & 0xf;
      switch (bank & 0x3) {
      case 0x0:
	/* not fitted */
	fitted = FALSE;
	pDevCtx->info.bankInfo[i].type = 0;
	break;
      case 0x1:
	/* ZBT flow-through */
	fitted = TRUE;
	pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_ZBTFT;
	break;
      case 0x2:
	/* ZBT pipelined */
	fitted = TRUE;
	pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_ZBTP;
	break;
      case 0x3:
	/* ZBT flow-through + pipelined (selectable) */
	fitted = TRUE;
	pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_ZBTFT | ADMXRC2_RAM_ZBTP;
	break;
      }
      pDevCtx->info.bankInfo[i].width = 32;
      if (!fitted) {
	pDevCtx->info.bankInfo[i].size = 0;
      } else {
	switch ((bank >> 2) & 0x3) {
	case 0x0:
	  /* 8Mbit */
	  pDevCtx->info.bankInfo[i].size = 256*1024;
	  break;
	case 0x1:
	  /* 16Mbit */
	  pDevCtx->info.bankInfo[i].size = 512*1024;
	  break;
	case 0x2:
	  /* 32Mbit */
	  pDevCtx->info.bankInfo[i].size = 1024*1024;
	  break;
	case 0x3:
	  /* reserved */
	  pDevCtx->info.bankInfo[i].size = 0;
	  fitted = FALSE;
	  break;
	}
      }
      pDevCtx->info.bankInfo[i].fitted = fitted;
      if (fitted) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (1 << i);
      }
    }
  }
  
  getSpaceInfoPci9656(pDevCtx);

  return MethodResult_Success;
}

MethodResult
methodInitCardInfoXrc4lx(
  DeviceContext* pDevCtx)
{
  os_uint32 data32;
  os_uint32 bankInfo;
  unsigned int i;
  
  os_dbgprint(3, ("methodInitCardInfoXrc4lx: entered\n"));

  pDevCtx->info.adapterLimit = 0x200;
  pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC4LX;
  pDevCtx->info.cardInfo.numClock = 2;
  pDevCtx->info.cardInfo.numDmaChan = pDevCtx->dma.info.numChan;
  pDevCtx->clock.lclkMin = admxrcDriverParams.model.xrc4lx.lclkMin;
  pDevCtx->clock.lclkMax = admxrcDriverParams.model.xrc4lx.lclkMax;
  
  /*
  ** Location 0x36: modifications word
  **
  ** The modifications word is interpreted in a board
  ** specific manner.
  */
  pDevCtx->info.modifications = 0x0;
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x36U, &data32);
  
  /* Location 0x37: clock oscillator type (ie. frequency) */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x37U, &data32);
  pDevCtx->info.clockType = (os_uint16) data32;
  
  /* Location 0x38: flash type (ie. device) - IGNORED */
  
  /* Location 0x3B: card ID */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3bU, &data32);
  pDevCtx->info.cardInfo.cardId = (ADMXRC2_CARDID) data32;
  
  /* Location 0x3C: fpga type (ie. device) */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3cU, &data32);
  pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE) data32;
  strcpy(pDevCtx->info.fpgaInfo[0].package, "FF1148");
  pDevCtx->info.fpgaInfo[0].type = pDevCtx->info.cardInfo.fpgaType;
  
  /* Location 0x3D: serial number low 16 bits */
  /* Location 0x3E: serial number high 16 bits */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3dU, &data32);
  pDevCtx->info.cardInfo.serialNum = data32 & 0xffffU;
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3eU, &data32);
  pDevCtx->info.cardInfo.serialNum |= (data32 & 0xffffU) << 16;
  
  /*
  ** Location 0x3F:
  **    low 8 bits = logic rev
  **    high 8 bits = board rev
  */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3fU, &data32);
  pDevCtx->info.cardInfo.boardRevision = (os_uint8) ((data32 >> 8) & 0xffU);
  pDevCtx->info.cardInfo.logicRevision = (os_uint8) ((data32 >> 0) & 0xffU);
  
  /* Locations 0x39 & 0x3A: SRAM presence info */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x39U, &data32);
  bankInfo = data32 & 0xffffU;
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3aU, &data32);
  bankInfo |= (data32 & 0xffff) << 16;
  
  {
    /*
    ** ADM-XRC4LX: get bank size/presence info
    */
    os_uint32 bank;
    os_boolean fitted = FALSE;
    
    pDevCtx->info.cardInfo.numRamBank = 6;
    pDevCtx->info.cardInfo.ramBanksFitted = 0;
    for (i = 0; i < pDevCtx->info.cardInfo.numRamBank; i++) {
      bank = (bankInfo >> (i << 2)) & 0xf;
      switch (bank & 0x3) {
      case 0x0:
	/* not fitted */
	fitted = FALSE;
	pDevCtx->info.bankInfo[i].type = 0;
	break;
      case 0x1:
	/* ZBT flow-through */
	fitted = TRUE;
	pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_ZBTFT;
	break;
      case 0x2:
	/* ZBT pipelined */
	fitted = TRUE;
	pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_ZBTP;
	break;
      case 0x3:
	/* ZBT flow-through + pipelined (selectable) */
	fitted = TRUE;
	pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_ZBTFT | ADMXRC2_RAM_ZBTP;
	break;
      }
      pDevCtx->info.bankInfo[i].width = 32;
      if (!fitted) {
	pDevCtx->info.bankInfo[i].size = 0;
      } else {
	switch ((bank >> 2) & 0x3) {
	case 0x0:
	    /* 8Mbit */
	  pDevCtx->info.bankInfo[i].size = 256*1024;
	  break;
	case 0x1:
	  /* 16Mbit */
	  pDevCtx->info.bankInfo[i].size = 512*1024;
	  break;
	case 0x2:
	  /* 32Mbit */
	  pDevCtx->info.bankInfo[i].size = 1024*1024;
	  break;
	case 0x3:
	  /* 64Mbit */
	  pDevCtx->info.bankInfo[i].size = 2048*1024;
	  break;
	}
      }
      pDevCtx->info.bankInfo[i].fitted = fitted;
      if (fitted) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (1 << i);
      }
    }
  }
  
  getSpaceInfoPci9656(pDevCtx);

  return MethodResult_Success;
}

MethodResult
methodInitCardInfoXrc4sx(
  DeviceContext* pDevCtx)
{
  os_uint32 data32;
  os_uint32 bankInfo;
  unsigned int	i;

  os_dbgprint(3, ("methodInitCardInfoXrc4sx: entered\n"));

  pDevCtx->info.adapterLimit = 0x200;
  
  pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC4SX;
  pDevCtx->info.cardInfo.numClock = 2;
  pDevCtx->info.cardInfo.numDmaChan = pDevCtx->dma.info.numChan;
  pDevCtx->clock.lclkMin = admxrcDriverParams.model.xrc4sx.lclkMin;
  pDevCtx->clock.lclkMax = admxrcDriverParams.model.xrc4sx.lclkMax;
  
  /*
  ** Location 0x36: modifications word
  **
  ** The modifications word is interpreted in a board
  ** specific manner.
  */
  pDevCtx->info.modifications = 0x0;
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x36U, &data32);
  
  /* Location 0x37: clock oscillator type (ie. frequency) */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x37U, &data32);
  pDevCtx->info.clockType = (os_uint16) data32;
  
  /* Location 0x38: flash type (ie. device) - IGNORED */
  
  /* Location 0x3B: card ID */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3bU, &data32);
  pDevCtx->info.cardInfo.cardId = (ADMXRC2_CARDID) data32;
  
  /* Location 0x3C: fpga type (ie. device) */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3cU, &data32);
  pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE) data32;
  strcpy(pDevCtx->info.fpgaInfo[0].package, "FF1148");
  pDevCtx->info.fpgaInfo[0].type = pDevCtx->info.cardInfo.fpgaType;
  
  /* Location 0x3D: serial number low 16 bits */
  /* Location 0x3E: serial number high 16 bits */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3dU, &data32);
  pDevCtx->info.cardInfo.serialNum = data32 & 0xffffU;
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3eU, &data32);
  pDevCtx->info.cardInfo.serialNum |= (data32 & 0xffffU) << 16;
  
  /*
  ** Location 0x3F:
  **    low 8 bits = logic rev
  **    high 8 bits = board rev
  */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3fU, &data32);
  pDevCtx->info.cardInfo.boardRevision = (os_uint8) ((data32 >> 8) & 0xffU);
  pDevCtx->info.cardInfo.logicRevision = (os_uint8) ((data32 >> 0) & 0xffU);
  
  /* Locations 0x39 & 0x3A: SRAM presence info */
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x39U, &data32);
  bankInfo = data32 & 0xffffU;
  pDevCtx->methods.configMem(pDevCtx, FALSE, 0x3aU, &data32);
  bankInfo |= (data32 & 0xffff) << 16;
  
  {
    /*
    ** ADM-XRC4SX: get bank size/presence info
    */
    os_uint32 bank;
    os_boolean fitted = FALSE;
    
    pDevCtx->info.cardInfo.numRamBank = 4;
    pDevCtx->info.cardInfo.ramBanksFitted = 0;
    for (i = 0; i < pDevCtx->info.cardInfo.numRamBank; i++) {
      bank = (bankInfo >> (i << 2)) & 0xf;
      switch (bank & 0x3) {
      case 0x0:
	/* not fitted */
	fitted = FALSE;
	pDevCtx->info.bankInfo[i].type = 0;
	break;
      case 0x1:
	/* ZBT flow-through */
	fitted = TRUE;
	pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_ZBTFT;
	break;
      case 0x2:
	/* ZBT pipelined */
	fitted = TRUE;
	pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_ZBTP;
	break;
      case 0x3:
	/* ZBT flow-through + pipelined (selectable) */
	fitted = TRUE;
	pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_ZBTFT | ADMXRC2_RAM_ZBTP;
	break;
      }
      pDevCtx->info.bankInfo[i].width = 32;
      if (!fitted) {
	pDevCtx->info.bankInfo[i].size = 0;
      } else {
	switch ((bank >> 2) & 0x3) {
	case 0x0:
	  /* 8Mbit */
	  pDevCtx->info.bankInfo[i].size = 256*1024;
	  break;
	case 0x1:
	  /* 16Mbit */
	  pDevCtx->info.bankInfo[i].size = 512*1024;
	  break;
	case 0x2:
	  /* 32Mbit */
	  pDevCtx->info.bankInfo[i].size = 1024*1024;
	  break;
	case 0x3:
	  /* 64Mbit */
	  pDevCtx->info.bankInfo[i].size = 2048*1024;
	  break;
	}
      }
      pDevCtx->info.bankInfo[i].fitted = fitted;
      if (fitted) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (1 << i);
      }
    }
  }
  
  getSpaceInfoPci9656(pDevCtx);

  return MethodResult_Success;
}

MethodResult
methodInitCardInfoXrce4fx(
  DeviceContext* pDevCtx)
{
  XRCE4FXCONFIGREV0 config;
  unsigned int i;
  os_boolean bConfigFound;
  
  os_dbgprint(3, ("methodInitCardInfoXrce4fx: entered\n"));
  
  pDevCtx->info.adapterLimit = 0x100;
  pDevCtx->clock.lclkMin = admxrcDriverParams.model.xrce4fx.lclkMin;
  pDevCtx->clock.lclkMax = admxrcDriverParams.model.xrce4fx.lclkMax;
  
  bConfigFound = FALSE;
  if (!admxrcDriverParams.model.xrce4fx.unlockFlash) {
    MethodResult result;
    
    result = readFlash(pDevCtx, 0, sizeof(config), &config);
    if (result == MethodResult_Success) {
      os_uint16 version;

      version = os_le16_to_cpu(config.version);
      switch (version) {
      case 0:
	bConfigFound = TRUE;
	break;
	
      default:
	/* Unrecognised data structure */
	break;
      }
    }
  }
  
  if (!bConfigFound) {
    os_dbgprint(0, ("*** methodInitCardInfoXrce4fx: no configuration information found\n"));
  }

  if (bConfigFound) {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADPEXRC4FX;
    pDevCtx->info.cardInfo.numClock = 2;
    pDevCtx->info.cardInfo.numDmaChan = pDevCtx->dma.info.numChan;
    pDevCtx->info.cardInfo.cardId = os_le32_to_cpu(config.cardId);
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE) os_le16_to_cpu(config.fpgaType);
    pDevCtx->info.cardInfo.serialNum = os_le32_to_cpu(config.serialNumber);
    pDevCtx->info.cardInfo.boardRevision = os_le8_to_cpu(config.pcbRev);
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 4;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = os_le32_to_cpu(config.modifications);
    pDevCtx->info.clockType = 0;
    pDevCtx->clock.refClk.xrce4fx.lclk = os_le32_to_cpu(config.lclkRef);
    pDevCtx->clock.refClk.xrce4fx.mclk = os_le32_to_cpu(config.mclkRef);
    for (i = 0; i < 4; i++) {
      os_uint8 order;

      pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_SDRAM_DDR2;
      pDevCtx->info.bankInfo[i].width = 64;
      order = os_le8_to_cpu(config.sdram[i].order);
      if (order) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << i);
	pDevCtx->info.bankInfo[i].fitted = TRUE;
	pDevCtx->info.bankInfo[i].size = 1 << order;
      } else {
	pDevCtx->info.bankInfo[i].fitted = FALSE;
	pDevCtx->info.bankInfo[i].size = 0;
      }
    }
  } else {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADPEXRC4FX;
    pDevCtx->info.cardInfo.numClock = 1;
    pDevCtx->info.cardInfo.numDmaChan = 0;
    pDevCtx->info.cardInfo.cardId = 0xdeadU;
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE)ADMXRC2_FPGA_4VFX100;
    pDevCtx->info.cardInfo.serialNum = 0xcafefaceU;
    pDevCtx->info.cardInfo.boardRevision = 0x10;
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 0;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = 0x0;
    pDevCtx->info.clockType = 0;
  }
  strcpy(pDevCtx->info.fpgaInfo[0].package, "FF1517");
  pDevCtx->info.fpgaInfo[0].type = pDevCtx->info.cardInfo.fpgaType;
  
  getSpaceInfoAdb1(pDevCtx);

  return bConfigFound ? MethodResult_Success : MethodResult_NotPresent;
}

MethodResult
methodInitCardInfoXrc4fx(
  DeviceContext* pDevCtx)
{
  XRC4FXCONFIGREV0 config;
  unsigned int i;
  os_boolean bConfigFound;
  
  os_dbgprint(3, ("methodInitCardInfoXrc4fx: entered\n"));
  
  pDevCtx->info.adapterLimit = 0x100;
  pDevCtx->clock.lclkMin = admxrcDriverParams.model.xrc4fx.lclkMin;
  pDevCtx->clock.lclkMax = admxrcDriverParams.model.xrc4fx.lclkMax;
  
  bConfigFound = FALSE;
  if (!admxrcDriverParams.model.xrc4fx.unlockFlash) {
    MethodResult result;
    
    result = readFlash(pDevCtx, 0, sizeof(config), &config);
    if (result == MethodResult_Success) {
      os_uint16 version;

      version = os_le16_to_cpu(config.version);
      switch (version) {
      case 0:
	bConfigFound = TRUE;
	break;
	
      default:
	/* Unrecognised data structure */
	break;
      }
    }
  }
  
  if (!bConfigFound) {
    os_dbgprint(0, ("*** methodInitCardInfoXrc4fx: no configuration information found\n"));
  }

  if (bConfigFound) {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC4FX;
    pDevCtx->info.cardInfo.numClock = 2;
    pDevCtx->info.cardInfo.numDmaChan = pDevCtx->dma.info.numChan;
    pDevCtx->info.cardInfo.cardId = os_le32_to_cpu(config.cardId);
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE) os_le16_to_cpu(config.fpgaType);
    pDevCtx->info.cardInfo.serialNum = os_le32_to_cpu(config.serialNumber);
    pDevCtx->info.cardInfo.boardRevision = os_le8_to_cpu(config.pcbRev);
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 4;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = os_le32_to_cpu(config.modifications);
    pDevCtx->info.clockType = 0;
    pDevCtx->clock.refClk.xrc4fx.lclk = os_le32_to_cpu(config.lclkRef);
    pDevCtx->clock.refClk.xrc4fx.mclkA = os_le32_to_cpu(config.mclkARef);
    pDevCtx->clock.refClk.xrc4fx.mclkB = os_le32_to_cpu(config.mclkBRef);
    for (i = 0; i < 4; i++) {
      os_uint8 order;

      pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_SDRAM_DDR2;
      pDevCtx->info.bankInfo[i].width = 64;
      order = os_le8_to_cpu(config.sdram[i].order);
      if (order) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << i);
	pDevCtx->info.bankInfo[i].fitted = TRUE;
	pDevCtx->info.bankInfo[i].size = 1 << order;
      } else {
	pDevCtx->info.bankInfo[i].fitted = FALSE;
	pDevCtx->info.bankInfo[i].size = 0;
      }
    }
  } else {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC4FX;
    pDevCtx->info.cardInfo.numClock = 2;
    pDevCtx->info.cardInfo.numDmaChan = 0;
    pDevCtx->info.cardInfo.cardId = 0xdeadU;
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE) ADMXRC2_FPGA_4VFX100;
    pDevCtx->info.cardInfo.serialNum = 0xcafefaceU;
    pDevCtx->info.cardInfo.boardRevision = 0x10;
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 0;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = 0x0;
    pDevCtx->info.clockType = 0;
  }
  strcpy(pDevCtx->info.fpgaInfo[0].package, "FF1517");
  pDevCtx->info.fpgaInfo[0].type = pDevCtx->info.cardInfo.fpgaType;
  
  getSpaceInfoAdb1(pDevCtx);

  return bConfigFound ? MethodResult_Success : MethodResult_NotPresent;
}

MethodResult
methodInitCardInfoXrc5lx(
  DeviceContext* pDevCtx)
{
  XRC5LXCONFIGREV0 config;
  unsigned int i;
  os_boolean bConfigFound;
  
  os_dbgprint(3, ("methodInitCardInfoXrc5lx: entered\n"));
  
  pDevCtx->info.adapterLimit = 0x100;
  pDevCtx->clock.lclkMin = admxrcDriverParams.model.xrc5lx.lclkMin;
  pDevCtx->clock.lclkMax = admxrcDriverParams.model.xrc5lx.lclkMax;
  
  bConfigFound = FALSE;
  if (!admxrcDriverParams.model.xrc5lx.unlockFlash) {
    MethodResult result;
    
    result = readFlash(pDevCtx, 0, sizeof(config), &config);
    if (result == MethodResult_Success) {
      os_uint16 version;

      version = os_le16_to_cpu(config.version);
      switch (version) {
      case 0:
	bConfigFound = TRUE;
	break;
	
      default:
	/* Unrecognised data structure */
	break;
      }
    }
  }
  
  if (!bConfigFound) {
    os_dbgprint(0, ("*** methodInitCardInfoXrc5lx: no configuration information found\n"));
  }

  if (bConfigFound) {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC5LX;
    pDevCtx->info.cardInfo.numClock = 2;
    pDevCtx->info.cardInfo.numDmaChan = pDevCtx->dma.info.numChan;
    pDevCtx->info.cardInfo.cardId = os_le32_to_cpu(config.cardId);
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE) os_le16_to_cpu(config.fpgaType);
    pDevCtx->info.cardInfo.serialNum = os_le32_to_cpu(config.serialNumber);
    pDevCtx->info.cardInfo.boardRevision = os_le8_to_cpu(config.pcbRev);
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 4;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = os_le32_to_cpu(config.modifications);
    pDevCtx->info.clockType = 0;
    pDevCtx->clock.refClk.xrc5lx.lclk = os_le32_to_cpu(config.lclkRef);
    pDevCtx->clock.refClk.xrc5lx.mclk = os_le32_to_cpu(config.mclkRef);
    for (i = 0; i < 4; i++) {
      os_uint8 order;

      pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_SDRAM_DDR2;
      pDevCtx->info.bankInfo[i].width = 64;
      order = os_le8_to_cpu(config.sdram[i].order);
      if (order) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << i);
	pDevCtx->info.bankInfo[i].fitted = TRUE;
	pDevCtx->info.bankInfo[i].size = 1 << order;
      } else {
	pDevCtx->info.bankInfo[i].fitted = FALSE;
	pDevCtx->info.bankInfo[i].size = 0;
      }
    }
  } else {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC5LX;
    pDevCtx->info.cardInfo.numClock = 2;
    pDevCtx->info.cardInfo.numDmaChan = 0;
    pDevCtx->info.cardInfo.cardId = 0xdeadU;
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE) ADMXRC2_FPGA_5VLX110;
    pDevCtx->info.cardInfo.serialNum = 0xcafefaceU;
    pDevCtx->info.cardInfo.boardRevision = 0x10;
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 0;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = 0x0;
    pDevCtx->info.clockType = 0;
  }
  strcpy(pDevCtx->info.fpgaInfo[0].package, "FF1153");
  pDevCtx->info.fpgaInfo[0].type = pDevCtx->info.cardInfo.fpgaType;
  
  getSpaceInfoAdb1(pDevCtx);

  return bConfigFound ? MethodResult_Success : MethodResult_NotPresent;
}

MethodResult
methodInitCardInfoXrc5t1(
  DeviceContext* pDevCtx)
{
  XRC5T1CONFIGREV0 config;
  unsigned int i;
  os_boolean bConfigFound;
  
  os_dbgprint(3, ("methodInitCardInfoXrc5t1: entered\n"));
  
  pDevCtx->info.adapterLimit = 0x100;
  pDevCtx->clock.lclkMin = admxrcDriverParams.model.xrc5t1.lclkMin;
  pDevCtx->clock.lclkMax = admxrcDriverParams.model.xrc5t1.lclkMax;
  
  bConfigFound = FALSE;
  if (!admxrcDriverParams.model.xrc5t1.unlockFlash) {
    MethodResult result;
    
    result = readFlash(pDevCtx, 0, sizeof(config), &config);
    if (result == MethodResult_Success) {
      os_uint16 version;

      version = os_le16_to_cpu(config.version);
      switch (version) {
      case 0:
	bConfigFound = TRUE;
	break;
	
      default:
	/* Unrecognised data structure */
	break;
      }
    }
  }
  
  if (!bConfigFound) {
    os_dbgprint(0, ("*** methodInitCardInfoXrc5t1: no configuration information found\n"));
  }

  if (bConfigFound) {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC5T1;
    pDevCtx->info.cardInfo.numClock = 2;
    pDevCtx->info.cardInfo.numDmaChan = pDevCtx->dma.info.numChan;
    pDevCtx->info.cardInfo.cardId = os_le32_to_cpu(config.cardId);
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE)os_le16_to_cpu(config.fpgaType);
    pDevCtx->info.cardInfo.serialNum = os_le32_to_cpu(config.serialNumber);
    pDevCtx->info.cardInfo.boardRevision = os_le8_to_cpu(config.pcbRev);
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 3;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = os_le32_to_cpu(config.modifications);
    pDevCtx->info.clockType = 0;
    pDevCtx->clock.refClk.xrc4fx.lclk = os_le32_to_cpu(config.lclkRef);
    pDevCtx->clock.refClk.xrc4fx.mclkA = os_le32_to_cpu(config.mclkARef);
    pDevCtx->clock.refClk.xrc4fx.mclkB = os_le32_to_cpu(config.mclkBRef);
    for (i = 0; i < 2; i++) {
      os_uint8 order;

      pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_SDRAM_DDR2;
      pDevCtx->info.bankInfo[i].width = 64;
      order = os_le8_to_cpu(config.sdram[i].order);
      if (order) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << i);
	pDevCtx->info.bankInfo[i].fitted = TRUE;
	pDevCtx->info.bankInfo[i].size = 1 << order;
      } else {
	pDevCtx->info.bankInfo[i].fitted = FALSE;
	pDevCtx->info.bankInfo[i].size = 0;
      }
    }
    for (i = 0; i < 1; i++) {
      os_uint8 order;

      pDevCtx->info.bankInfo[i + 2].type = ADMXRC2_RAM_SRAM_DDR2;
      pDevCtx->info.bankInfo[i + 2].width = 36;
      order = os_le8_to_cpu(config.ssram[i].order);
      if (order) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << (i + 2));
	pDevCtx->info.bankInfo[i + 2].fitted = TRUE;
	pDevCtx->info.bankInfo[i + 2].size = 1 << order;
      } else {
	pDevCtx->info.bankInfo[i + 2].fitted = FALSE;
	pDevCtx->info.bankInfo[i + 2].size = 0;
      }
    }
  } else {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC5T1;
    pDevCtx->info.cardInfo.numClock = 2;
    pDevCtx->info.cardInfo.numDmaChan = 0;
    pDevCtx->info.cardInfo.cardId = 0xdeadU;
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE) ADMXRC2_FPGA_5VLX110T;
    pDevCtx->info.cardInfo.serialNum = 0xcafefaceU;
    pDevCtx->info.cardInfo.boardRevision = 0x10;
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 0;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = 0x0;
    pDevCtx->info.clockType = 0;
  }
  strcpy(pDevCtx->info.fpgaInfo[0].package, "FF1136");
  pDevCtx->info.fpgaInfo[0].type = pDevCtx->info.cardInfo.fpgaType;
  
  getSpaceInfoAdb1(pDevCtx);

  return bConfigFound ? MethodResult_Success : MethodResult_NotPresent;
}

MethodResult
methodInitCardInfoXrc5t1VpdBram(
  DeviceContext* pDevCtx)
{
  union {
    XRC5T1CONFIGREV0 asStruct;
    os_uint32 asArray[(sizeof(XRC5T1CONFIGREV0) + sizeof(os_uint32) - 1) / sizeof(os_uint32)];
  } config;
  unsigned int i;
  os_boolean bConfigFound;
  
  os_dbgprint(3, ("methodInitCardInfoXrc5t1VpdBram: entered\n"));
  
  pDevCtx->info.adapterLimit = 0x100;
  pDevCtx->clock.lclkMin = admxrcDriverParams.model.xrc5t1.lclkMin;
  pDevCtx->clock.lclkMax = admxrcDriverParams.model.xrc5t1.lclkMax;
  
  bConfigFound = FALSE;
  /* Read VPD out of BlockRAM in special version of ADB1 */
  for (i = 0; i < sizeof(config) / sizeof(os_uint32); i++) {
    pDevCtx->methods.configMem(pDevCtx, FALSE, i, &config.asArray[i]);
  }
  switch (config.asStruct.version) {
  case 0:
    bConfigFound = TRUE;
    break;

  default:
    /* Unrecognised data structure */
    break;
  }

  if (!bConfigFound) {
    os_dbgprint(0, ("*** methodInitCardInfoXrc5t1VpdBram: no configuration information found\n"));
  }

  if (bConfigFound) {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC5T1;
    pDevCtx->info.cardInfo.numClock = 2;
    pDevCtx->info.cardInfo.numDmaChan = pDevCtx->dma.info.numChan;
    pDevCtx->info.cardInfo.cardId = os_le32_to_cpu(config.asStruct.cardId);
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE) os_le16_to_cpu(config.asStruct.fpgaType);
    pDevCtx->info.cardInfo.serialNum = os_le32_to_cpu(config.asStruct.serialNumber);
    pDevCtx->info.cardInfo.boardRevision = os_le8_to_cpu(config.asStruct.pcbRev);
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 3;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = os_le32_to_cpu(config.asStruct.modifications);
    pDevCtx->info.clockType = 0;
    pDevCtx->clock.refClk.xrc4fx.lclk = os_le32_to_cpu(config.asStruct.lclkRef);
    pDevCtx->clock.refClk.xrc4fx.mclkA = os_le32_to_cpu(config.asStruct.mclkARef);
    pDevCtx->clock.refClk.xrc4fx.mclkB = os_le32_to_cpu(config.asStruct.mclkBRef);
    for (i = 0; i < 2; i++) {
      os_uint8 order;

      pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_SDRAM_DDR2;
      pDevCtx->info.bankInfo[i].width = 64;
      order = os_le8_to_cpu(config.asStruct.sdram[i].order);
      if (order) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << i);
	pDevCtx->info.bankInfo[i].fitted = TRUE;
	pDevCtx->info.bankInfo[i].size = 1 << order;
      } else {
	pDevCtx->info.bankInfo[i].fitted = FALSE;
	pDevCtx->info.bankInfo[i].size = 0;
      }
    }
    for (i = 0; i < 1; i++) {
      os_uint8 order;

      pDevCtx->info.bankInfo[i + 2].type = ADMXRC2_RAM_SRAM_DDR2;
      pDevCtx->info.bankInfo[i + 2].width = 36;
      order = os_le8_to_cpu(config.asStruct.ssram[i].order);
      if (order) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << (i + 2));
	pDevCtx->info.bankInfo[i + 2].fitted = TRUE;
	pDevCtx->info.bankInfo[i + 2].size = 1 << order;
      } else {
	pDevCtx->info.bankInfo[i + 2].fitted = FALSE;
	pDevCtx->info.bankInfo[i + 2].size = 0;
      }
    }
  } else {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC5T1;
    pDevCtx->info.cardInfo.numClock = 2;
    pDevCtx->info.cardInfo.numDmaChan = 0;
    pDevCtx->info.cardInfo.cardId = 0xdeadU;
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE) ADMXRC2_FPGA_5VLX110T;
    pDevCtx->info.cardInfo.serialNum = 0xcafefaceU;
    pDevCtx->info.cardInfo.boardRevision = 0x10;
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 0;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = 0x0;
    pDevCtx->info.clockType = 0;
  }
  strcpy(pDevCtx->info.fpgaInfo[0].package, "FF1136");
  pDevCtx->info.fpgaInfo[0].type = pDevCtx->info.cardInfo.fpgaType;
  
  getSpaceInfoAdb1(pDevCtx);

  return bConfigFound ? MethodResult_Success : MethodResult_NotPresent;
}

MethodResult
methodInitCardInfoXrc5lxa(
  DeviceContext* pDevCtx)
{
  XRC5LXACONFIGREV0 config;
  unsigned int i;
  os_boolean bConfigFound;
  
  os_dbgprint(3, ("methodInitCardInfoXrc5lxa: entered\n"));
  
  pDevCtx->info.adapterLimit = 0x100;
  pDevCtx->clock.lclkMin = admxrcDriverParams.model.xrc5lxa.lclkMin;
  pDevCtx->clock.lclkMax = admxrcDriverParams.model.xrc5lxa.lclkMax;
  
  bConfigFound = FALSE;
  if (!admxrcDriverParams.model.xrc5lxa.unlockFlash) {
    MethodResult result;
    
    result = readFlash(pDevCtx, 0, sizeof(config), &config);
    if (result == MethodResult_Success) {
      os_uint16 version;

      version = os_le16_to_cpu(config.version);
      switch (version) {
      case 0:
	bConfigFound = TRUE;
	break;
	
      default:
	/* Unrecognised data structure */
	break;
      }
    }
  }
  
  if (!bConfigFound) {
    os_dbgprint(0, ("*** methodInitCardInfoXrc5lx: no configuration information found\n"));
  }

  if (bConfigFound) {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC5LX;
    pDevCtx->info.cardInfo.numClock = 2;
    pDevCtx->info.cardInfo.numDmaChan = pDevCtx->dma.info.numChan;
    pDevCtx->info.cardInfo.cardId = os_le32_to_cpu(config.cardId);
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE) os_le16_to_cpu(config.fpgaType);
    pDevCtx->info.cardInfo.serialNum = os_le32_to_cpu(config.serialNumber);
    pDevCtx->info.cardInfo.boardRevision = os_le8_to_cpu(config.pcbRev);
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 4;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = os_le32_to_cpu(config.modifications);
    pDevCtx->info.clockType = 0;
    pDevCtx->clock.refClk.xrc5lxa.lclk = os_le32_to_cpu(config.lclkRef);
    pDevCtx->clock.refClk.xrc5lxa.mclk = os_le32_to_cpu(config.mclkRef);
    for (i = 0; i < 4; i++) {
      os_uint8 order;

      pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_SDRAM_DDR2;
      pDevCtx->info.bankInfo[i].width = 64;
      order = os_le8_to_cpu(config.sdram[i].order);
      if (order) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << i);
	pDevCtx->info.bankInfo[i].fitted = TRUE;
	pDevCtx->info.bankInfo[i].size = 1 << order;
      } else {
	pDevCtx->info.bankInfo[i].fitted = FALSE;
	pDevCtx->info.bankInfo[i].size = 0;
      }
    }
  } else {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC5T1;
    pDevCtx->info.cardInfo.numClock = 2;
    pDevCtx->info.cardInfo.numDmaChan = 0;
    pDevCtx->info.cardInfo.cardId = 0xdeadU;
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE) ADMXRC2_FPGA_5VLX110T;
    pDevCtx->info.cardInfo.serialNum = 0xcafefaceU;
    pDevCtx->info.cardInfo.boardRevision = 0x10;
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 0;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = 0x0;
    pDevCtx->info.clockType = 0;
  }
  strcpy(pDevCtx->info.fpgaInfo[0].package, "FF1153");
  pDevCtx->info.fpgaInfo[0].type = pDevCtx->info.cardInfo.fpgaType;
  
  getSpaceInfoAdb1(pDevCtx);

  return bConfigFound ? MethodResult_Success : MethodResult_NotPresent;
}

MethodResult
methodInitCardInfoXrcp4lx(
  DeviceContext* pDevCtx)
{
  XRCP4LXCONFIGREV0 config;
  unsigned int i;
  os_boolean bConfigFound;
  
  os_dbgprint(3, ("methodInitCardInfoXrcp4lx: entered\n"));
  
  pDevCtx->info.adapterLimit = 0x100;
  pDevCtx->clock.lclkMin = admxrcDriverParams.model.xrcp4lx.lclkMin;
  pDevCtx->clock.lclkMax = admxrcDriverParams.model.xrcp4lx.lclkMax;
  
  bConfigFound = FALSE;
  if (!admxrcDriverParams.model.xrcp4lx.unlockFlash) {
    MethodResult result;
    
    result = readFlash(pDevCtx, 0, sizeof(config), &config);
    if (result == MethodResult_Success) {
      os_uint16 version;

      version = os_le16_to_cpu(config.version);
      switch (version) {
      case 0:
	bConfigFound = TRUE;
	break;
	
      default:
	/* Unrecognised data structure */
	break;
      }
    }
  }
  
  if (!bConfigFound) {
    os_dbgprint(0, ("*** methodInitCardInfoXrcp4lx: no configuration information found\n"));
  }

  if (bConfigFound) {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADCPXRC4LX;
    pDevCtx->info.cardInfo.numClock = 0;
    pDevCtx->info.cardInfo.numDmaChan = pDevCtx->dma.info.numChan;
    pDevCtx->info.cardInfo.cardId = os_le32_to_cpu(config.cardId);
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE)os_le16_to_cpu(config.fpgaType);
    pDevCtx->info.cardInfo.serialNum = os_le32_to_cpu(config.serialNumber);
    pDevCtx->info.cardInfo.boardRevision = os_le8_to_cpu(config.pcbRev);
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 4;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = os_le32_to_cpu(config.modifications);
    pDevCtx->info.clockType = 0;
    for (i = 0; i < 2; i++) {
      os_uint8 order;

      pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_SDRAM_DDR2;
      pDevCtx->info.bankInfo[i].width = 32;
      order = os_le8_to_cpu(config.sdram[i].order);
      if (order) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << i);
	pDevCtx->info.bankInfo[i].fitted = TRUE;
	pDevCtx->info.bankInfo[i].size = 1 << order;
      } else {
	pDevCtx->info.bankInfo[i].fitted = FALSE;
	pDevCtx->info.bankInfo[i].size = 0;
      }
    }
    for (i = 0; i < 2; i++) {
      os_uint8 order;

      pDevCtx->info.bankInfo[i + 2].type = ADMXRC2_RAM_ZBTP;
      pDevCtx->info.bankInfo[i + 2].width = 32;
      order = os_le8_to_cpu(config.zbt[i].order);
      if (order) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << (i + 2));
	pDevCtx->info.bankInfo[i + 2].fitted = TRUE;
	pDevCtx->info.bankInfo[i + 2].size = 1 << order;
      } else {
	pDevCtx->info.bankInfo[i + 2].fitted = FALSE;
	pDevCtx->info.bankInfo[i + 2].size = 0;
      }
    }
  } else {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADCPXRC4LX;
    pDevCtx->info.cardInfo.numClock = 0;
    pDevCtx->info.cardInfo.numDmaChan = 0;
    pDevCtx->info.cardInfo.cardId = 0xdeadU;
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE)ADMXRC2_FPGA_4VLX80;
    pDevCtx->info.cardInfo.serialNum = 0xcafefaceU;
    pDevCtx->info.cardInfo.boardRevision = 0x10;
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 0;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = 0x0;
    pDevCtx->info.clockType = 0;
  }
  strcpy(pDevCtx->info.fpgaInfo[0].package, "FF1148");
  pDevCtx->info.fpgaInfo[0].type = pDevCtx->info.cardInfo.fpgaType;
  
  getSpaceInfoAdb1(pDevCtx);

  return bConfigFound ? MethodResult_Success : MethodResult_NotPresent;
}

MethodResult
methodInitCardInfoXrc5t2(
  DeviceContext* pDevCtx)
{
  XRC5T2CONFIGREV0 config;
  unsigned int i;
  os_boolean bConfigFound;
  
  os_dbgprint(3, ("methodInitCardInfoXrc5t2: entered\n"));
  
  pDevCtx->info.adapterLimit = 0x100;
  pDevCtx->clock.lclkMin = admxrcDriverParams.model.xrc5t2.lclkMin;
  pDevCtx->clock.lclkMax = admxrcDriverParams.model.xrc5t2.lclkMax;
  
  bConfigFound = FALSE;
  if (!admxrcDriverParams.model.xrc5t2.unlockFlash) {
    MethodResult result;
    
    result = readFlash(pDevCtx, 0, sizeof(config), &config);
    if (result == MethodResult_Success) {
      os_uint16 version;

      version = os_le16_to_cpu(config.version);
      switch (version) {
      case 0:
	bConfigFound = TRUE;
	break;
	
      default:
	/* Unrecognised data structure */
	break;
      }
    }
  }
  
  if (!bConfigFound) {
    os_dbgprint(0, ("*** methodInitCardInfoXrc5t2: no configuration information found\n"));
  }

  if (pDevCtx->devId.pci.subDevice == PCI_SUBDEVICE_ADATA_ADMXRC5T2ADV) {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC5T2ADV;
  } else if (pDevCtx->devId.pci.subDevice == PCI_SUBDEVICE_ADATA_ADMXRC5T2ADV6) {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC5T2ADV6;
  } else if (pDevCtx->devId.pci.subDevice == PCI_SUBDEVICE_ADATA_ADMXRC5T2ADVCC1) {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC5T2ADVCC1;
  } else {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC5T2;
  }
  pDevCtx->info.cardInfo.numClock = 2;
  pDevCtx->info.cardInfo.numSpace = 2;
  pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
  if (bConfigFound) {
    pDevCtx->info.cardInfo.numDmaChan = pDevCtx->dma.info.numChan;
    pDevCtx->info.cardInfo.cardId = os_le32_to_cpu(config.cardId);
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE)os_le16_to_cpu(config.fpgaType);
    pDevCtx->info.cardInfo.serialNum = os_le32_to_cpu(config.serialNumber);
    pDevCtx->info.cardInfo.boardRevision = os_le8_to_cpu(config.pcbRev);
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 6;
      pDevCtx->info.modifications = os_le32_to_cpu(config.modifications);
    pDevCtx->info.clockType = 0;
    pDevCtx->clock.refClk.xrc4fx.lclk = os_le32_to_cpu(config.lclkRef);
    pDevCtx->clock.refClk.xrc4fx.mclkA = os_le32_to_cpu(config.mclkARef);
    pDevCtx->clock.refClk.xrc4fx.mclkB = os_le32_to_cpu(config.mclkBRef);
    for (i = 0; i < 4; i++) {
      os_uint8 order;

      pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_SDRAM_DDR2;
      pDevCtx->info.bankInfo[i].width = 64;
      order = os_le8_to_cpu(config.sdram[i].order);
      if (order) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << i);
	pDevCtx->info.bankInfo[i].fitted = TRUE;
	pDevCtx->info.bankInfo[i].size = 1 << order;
      } else {
	pDevCtx->info.bankInfo[i].fitted = FALSE;
	pDevCtx->info.bankInfo[i].size = 0;
      }
    }
    for (i = 0; i < 2; i++) {
      os_uint8 order;

      pDevCtx->info.bankInfo[i + 4].type = ADMXRC2_RAM_SRAM_DDR2;
      pDevCtx->info.bankInfo[i + 4].width = 36;
      order = os_le8_to_cpu(config.ssram[i].order);
      if (order) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << (i + 4));
	pDevCtx->info.bankInfo[i + 4].fitted = TRUE;
	pDevCtx->info.bankInfo[i + 4].size = 1 << order;
      } else {
	pDevCtx->info.bankInfo[i + 4].fitted = FALSE;
	pDevCtx->info.bankInfo[i + 4].size = 0;
      }
    }
  } else {
    pDevCtx->info.cardInfo.numDmaChan = 0;
    pDevCtx->info.cardInfo.cardId = 0xdeadU;
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE)ADMXRC2_FPGA_5VLX110T;
    pDevCtx->info.cardInfo.serialNum = 0xcafefaceU;
    pDevCtx->info.cardInfo.boardRevision = 0x10;
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 0;
    pDevCtx->info.modifications = 0x0;
    pDevCtx->info.clockType = 0;
  }
  strcpy(pDevCtx->info.fpgaInfo[0].package, "FF1738");
  pDevCtx->info.fpgaInfo[0].type = pDevCtx->info.cardInfo.fpgaType;
  
  getSpaceInfoAdb1(pDevCtx);

  return bConfigFound ? MethodResult_Success : MethodResult_NotPresent;
}

MethodResult
methodInitCardInfoAmc5a2(
  DeviceContext* pDevCtx)
{
  AMC5A2CONFIGREV0 config;
  unsigned int i;
  os_boolean bConfigFound;
  
  os_dbgprint(3, ("methodInitCardInfoAmc5a2: entered\n"));
  
  pDevCtx->info.adapterLimit = 0x100;
  pDevCtx->clock.lclkMin = admxrcDriverParams.model.amc5a2.lclkMin;
  pDevCtx->clock.lclkMax = admxrcDriverParams.model.amc5a2.lclkMax;
  
  bConfigFound = FALSE;
  if (!admxrcDriverParams.model.amc5a2.unlockFlash) {
    MethodResult result;
    
    result = readFlash(pDevCtx, VPD_ADDR_ADMAMC5A2, sizeof(config), &config);
    if (result == MethodResult_Success) {
      switch (config.version) {
      case 0:
	bConfigFound = TRUE;
	break;
	
      default:
	/* Unrecognised data structure */
	break;
      }
    }
  }
  
  if (bConfigFound) {
    pDevCtx->info.cardInfo.boardType = config.boardType;
    pDevCtx->info.cardInfo.numClock = 1;
    pDevCtx->info.cardInfo.numDmaChan = pDevCtx->dma.info.numChan;
    pDevCtx->info.cardInfo.cardId = os_le32_to_cpu(config.cardId);
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE)os_le16_to_cpu(config.fpgaType);
    pDevCtx->info.cardInfo.serialNum = os_le32_to_cpu(config.serialNumber);
    pDevCtx->info.cardInfo.boardRevision = os_le8_to_cpu(config.pcbRev);
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 4;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = os_le16_to_cpu(config.modifications);
    pDevCtx->info.clockType = 0;
    pDevCtx->clock.refClk.amc5a2.lclk = os_le32_to_cpu(config.lclkRef);
    for (i = 0; i < 4; i++) {
      os_uint8 order;

      pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_SDRAM_DDR2;
      pDevCtx->info.bankInfo[i].width = 64;
      order = os_le8_to_cpu(config.sdram[i].order);
      if (order) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << i);
	pDevCtx->info.bankInfo[i].fitted = TRUE;
	pDevCtx->info.bankInfo[i].size = 1UL << order;
      } else {
	pDevCtx->info.bankInfo[i].fitted = FALSE;
	pDevCtx->info.bankInfo[i].size = 0;
      }
    }
  } else {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMAMC5A2;
    pDevCtx->info.cardInfo.numClock = 1;
    pDevCtx->info.cardInfo.numDmaChan = 0;
    pDevCtx->info.cardInfo.cardId = 0xdeadU;
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE)ADMXRC2_FPGA_5VLX330T;
    pDevCtx->info.cardInfo.serialNum = 0xcafefaceU;
    pDevCtx->info.cardInfo.boardRevision = 0x10;
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 0;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = 0x0;
    pDevCtx->info.clockType = 0;
    pDevCtx->clock.refClk.amc5a2.lclk = 25000000;
  }
  strcpy(pDevCtx->info.fpgaInfo[0].package, "FF1738");
  pDevCtx->info.fpgaInfo[0].type = pDevCtx->info.cardInfo.fpgaType;
  
  getSpaceInfoAdb1(pDevCtx);

  return MethodResult_Success;
}

MethodResult
methodInitCardInfoXrc5tz(
  DeviceContext* pDevCtx)
{
  XRC5TZCONFIGREV0 config;
  unsigned int i;
  os_boolean bConfigFound;
  
  os_dbgprint(3, ("methodInitCardInfoXrc5tz: entered\n"));
  
  pDevCtx->info.adapterLimit = 0x100;
  pDevCtx->clock.lclkMin = admxrcDriverParams.model.xrc5tz.lclkMin;
  pDevCtx->clock.lclkMax = admxrcDriverParams.model.xrc5tz.lclkMax;
  
  bConfigFound = FALSE;
  if (!admxrcDriverParams.model.xrc5tz.unlockFlash) {
    MethodResult result;
    
    result = readFlash(pDevCtx, 0, sizeof(config), &config);
    if (result == MethodResult_Success) {
      os_uint16 version;

      version = os_le16_to_cpu(config.version);
      switch (version) {
      case 0:
	bConfigFound = TRUE;
	break;
	
      default:
	/* Unrecognised data structure */
	break;
      }
    }
  }
  
  if (!bConfigFound) {
    os_dbgprint(0, ("*** methodInitCardInfoXrc5tz: no configuration information found\n"));
  }

  if (bConfigFound) {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC5TZ;
    pDevCtx->info.cardInfo.numClock = 2;
    pDevCtx->info.cardInfo.numDmaChan = pDevCtx->dma.info.numChan;
    pDevCtx->info.cardInfo.cardId = os_le32_to_cpu(config.cardId);
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE)os_le16_to_cpu(config.fpgaType);
    pDevCtx->info.cardInfo.serialNum = os_le32_to_cpu(config.serialNumber);
    pDevCtx->info.cardInfo.boardRevision = os_le8_to_cpu(config.pcbRev);
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 6;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = os_le32_to_cpu(config.modifications);
    pDevCtx->info.clockType = 0;
    pDevCtx->clock.refClk.xrc4fx.lclk = os_le32_to_cpu(config.lclkRef);
    pDevCtx->clock.refClk.xrc4fx.mclkA = os_le32_to_cpu(config.mclkARef);
    pDevCtx->clock.refClk.xrc4fx.mclkB = os_le32_to_cpu(config.mclkBRef);
    for (i = 0; i < 6; i++) {
      os_uint8 order;

      pDevCtx->info.bankInfo[i].type = config.zbt[i].type;
      pDevCtx->info.bankInfo[i].width = 32;
      order = os_le8_to_cpu(config.zbt[i].order);
      if (order) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << i);
	pDevCtx->info.bankInfo[i].fitted = TRUE;
	pDevCtx->info.bankInfo[i].size = 1 << order;
      } else {
	pDevCtx->info.bankInfo[i].fitted = FALSE;
	pDevCtx->info.bankInfo[i].size = 0;
      }
    }
  } else {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC5TZ;
    pDevCtx->info.cardInfo.numClock = 2;
    pDevCtx->info.cardInfo.numDmaChan = 0;
    pDevCtx->info.cardInfo.cardId = 0xdeadU;
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE) ADMXRC2_FPGA_5VLX110T;
    pDevCtx->info.cardInfo.serialNum = 0xcafefaceU;
    pDevCtx->info.cardInfo.boardRevision = 0x10;
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 0;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = 0x0;
    pDevCtx->info.clockType = 0;
  }
  strcpy(pDevCtx->info.fpgaInfo[0].package, "FF1738");
  pDevCtx->info.fpgaInfo[0].type = pDevCtx->info.cardInfo.fpgaType;
  
  getSpaceInfoAdb1(pDevCtx);

  return bConfigFound ? MethodResult_Success : MethodResult_NotPresent;
}

MethodResult
methodInitCardInfoBbp(
  DeviceContext* pDevCtx)
{
  BBPCONFIGREV0 config;
  unsigned int i;
  os_boolean bConfigFound;
  
  os_dbgprint(3, ("methodInitCardInfoBbp: entered\n"));
  
  pDevCtx->info.adapterLimit = 0x100;
  pDevCtx->clock.lclkMin = admxrcDriverParams.model.bbp.lclkMin;
  pDevCtx->clock.lclkMax = admxrcDriverParams.model.bbp.lclkMax;
  
  bConfigFound = FALSE;
  if (!admxrcDriverParams.model.bbp.unlockFlash) {
    MethodResult result;
    
    result = readFlash(pDevCtx, VPD_ADDR_ADCBBP, sizeof(config), &config);
    if (result == MethodResult_Success) {
      os_uint16 version;

      version = os_le16_to_cpu(config.version);
      switch (version) {
      case 0:
	bConfigFound = TRUE;
	break;
	
      default:
	/* Unrecognised data structure */
	break;
      }
    }
  }
  
  if (!bConfigFound) {
    os_dbgprint(0, ("*** methodInitCardInfoBbp: no configuration information found\n"));
  }

  if (bConfigFound) {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADCBBP;
    pDevCtx->info.cardInfo.numClock = 1;
    pDevCtx->info.cardInfo.numDmaChan = pDevCtx->dma.info.numChan;
    pDevCtx->info.cardInfo.cardId = os_le32_to_cpu(config.cardId);
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE)os_le16_to_cpu(config.fpgaType0);
    pDevCtx->info.fpgaInfo[0].type = pDevCtx->info.cardInfo.fpgaType;
    pDevCtx->info.fpgaInfo[1].type = (ADMXRC2_FPGA_TYPE)os_le16_to_cpu(config.fpgaType1);
    pDevCtx->info.cardInfo.serialNum = os_le32_to_cpu(config.serialNumber);
    pDevCtx->info.cardInfo.boardRevision = os_le8_to_cpu(config.pcbRev);
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 4;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = os_le32_to_cpu(config.modifications);
    pDevCtx->info.clockType = 0;
    pDevCtx->clock.refClk.bbp.lclk = os_le32_to_cpu(config.lclkRef);
    for (i = 0; i < 4; i++) {
      os_uint8 order;

      pDevCtx->info.bankInfo[i].type = config.zbt0[i].type;
      pDevCtx->info.bankInfo[i].width = 32;
      order = os_le8_to_cpu(config.zbt0[i].order);
      if (order) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << i);
	pDevCtx->info.bankInfo[i].fitted = TRUE;
	pDevCtx->info.bankInfo[i].size = 1 << order;
      } else {
	pDevCtx->info.bankInfo[i].fitted = FALSE;
	pDevCtx->info.bankInfo[i].size = 0;
      }
    }
  } else {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADCBBP;
    pDevCtx->info.cardInfo.numClock = 1;
    pDevCtx->info.cardInfo.numDmaChan = 0;
    pDevCtx->info.cardInfo.cardId = 0xdeadU;
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE)ADMXRC2_FPGA_5VLX110T;
    pDevCtx->info.fpgaInfo[0].type = pDevCtx->info.cardInfo.fpgaType;
    pDevCtx->info.fpgaInfo[1].type = (ADMXRC2_FPGA_TYPE)ADMXRC2_FPGA_5VLX110T;
    pDevCtx->info.cardInfo.serialNum = 0xcafefaceU;
    pDevCtx->info.cardInfo.boardRevision = 0x10;
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 0;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = 0x0;
    pDevCtx->info.clockType = 0;
  }
  strcpy(pDevCtx->info.fpgaInfo[0].package, "FF1136");
  strcpy(pDevCtx->info.fpgaInfo[1].package, "FF1738");
  
  getSpaceInfoAdb1(pDevCtx);

  return bConfigFound ? MethodResult_Success : MethodResult_NotPresent;
}

MethodResult
methodInitCardInfoXrc5tda1(
  DeviceContext* pDevCtx)
{
  XRC5TDA1CONFIGREV0 config;
  unsigned int i;
  os_boolean bConfigFound;
  
  os_dbgprint(3, ("methodInitCardInfoXrc5tda1: entered\n"));
  
  pDevCtx->info.adapterLimit = 0x100;
  pDevCtx->clock.lclkMin = admxrcDriverParams.model.xrc5tda1.lclkMin;
  pDevCtx->clock.lclkMax = admxrcDriverParams.model.xrc5tda1.lclkMax;
  
  bConfigFound = FALSE;
  if (!admxrcDriverParams.model.xrc5tda1.unlockFlash) {
    MethodResult result;
    
    result = readFlash(pDevCtx, 0, sizeof(config), &config);
    if (result == MethodResult_Success) {
      os_uint16 version;

      version = os_le16_to_cpu(config.version);
      switch (version) {
      case 0:
	bConfigFound = TRUE;
	break;
	
      default:
	/* Unrecognised data structure */
	break;
      }
    }
  }
  
  if (!bConfigFound) {
    os_dbgprint(0, ("*** methodInitCardInfoXrc5tda1: no configuration information found\n"));
  }

  if (bConfigFound) {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC5TDA1;
    pDevCtx->info.cardInfo.numClock = 2;
    pDevCtx->info.cardInfo.numDmaChan = pDevCtx->dma.info.numChan;
    pDevCtx->info.cardInfo.cardId = os_le32_to_cpu(config.cardId);
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE)os_le16_to_cpu(config.fpgaType);
    pDevCtx->info.cardInfo.serialNum = os_le32_to_cpu(config.serialNumber);
    pDevCtx->info.cardInfo.boardRevision = os_le8_to_cpu(config.pcbRev);
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 4;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = os_le32_to_cpu(config.modifications);
    pDevCtx->info.clockType = 0;
    pDevCtx->clock.refClk.xrc4fx.lclk = os_le32_to_cpu(config.lclkRef);
    pDevCtx->clock.refClk.xrc4fx.mclkA = os_le32_to_cpu(config.mclkARef);
    pDevCtx->clock.refClk.xrc4fx.mclkB = os_le32_to_cpu(config.mclkBRef);
    for (i = 0; i < 2; i++) {
      os_uint8 order;

      pDevCtx->info.bankInfo[i].type = ADMXRC2_RAM_SDRAM_DDR2;
      pDevCtx->info.bankInfo[i].width = 64;
      order = os_le8_to_cpu(config.sdram[i].order);
      if (order) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << i);
	pDevCtx->info.bankInfo[i].fitted = TRUE;
	pDevCtx->info.bankInfo[i].size = 1 << order;
      } else {
	pDevCtx->info.bankInfo[i].fitted = FALSE;
	pDevCtx->info.bankInfo[i].size = 0;
      }
    }
    for (i = 0; i < 2; i++) {
      os_uint8 order;

      pDevCtx->info.bankInfo[i + 2].type = ADMXRC2_RAM_SRAM_DDR2;
      pDevCtx->info.bankInfo[i + 2].width = 36;
      order = os_le8_to_cpu(config.ssram[i].order);
      if (order) {
	pDevCtx->info.cardInfo.ramBanksFitted |= (0x1U << (i + 2));
	pDevCtx->info.bankInfo[i + 2].fitted = TRUE;
	pDevCtx->info.bankInfo[i + 2].size = 1 << order;
      } else {
	pDevCtx->info.bankInfo[i + 2].fitted = FALSE;
	pDevCtx->info.bankInfo[i + 2].size = 0;
      }
    }
  } else {
    pDevCtx->info.cardInfo.boardType = ADMXRC2_BOARD_ADMXRC5TDA1;
    pDevCtx->info.cardInfo.numClock = 2;
    pDevCtx->info.cardInfo.numDmaChan = 0;
    pDevCtx->info.cardInfo.cardId = 0xdeadU;
    pDevCtx->info.cardInfo.fpgaType = (ADMXRC2_FPGA_TYPE)ADMXRC2_FPGA_5VLX110T;
    pDevCtx->info.cardInfo.serialNum = 0xcafefaceU;
    pDevCtx->info.cardInfo.boardRevision = 0x10;
    pDevCtx->info.cardInfo.logicRevision = pDevCtx->devId.pci.revision + 0x10U;
    pDevCtx->info.cardInfo.numRamBank = 0;
    pDevCtx->info.cardInfo.ramBanksFitted = 0x0;
    pDevCtx->info.cardInfo.numSpace = 2;
    pDevCtx->info.modifications = 0x0;
    pDevCtx->info.clockType = 0;
  }
  strcpy(pDevCtx->info.fpgaInfo[0].package, "FF1136");
  pDevCtx->info.fpgaInfo[0].type = pDevCtx->info.cardInfo.fpgaType;
  
  getSpaceInfoAdb1(pDevCtx);

  return bConfigFound ? MethodResult_Success : MethodResult_NotPresent;
}
