/*
** ADM-XRC Series Driver
**
** (C) Copyright 2001-2004 Alpha Data
**
** spaceconfig.c - IOCTL handlers for getting/setting local bus space
**                 configurations.
**
** ----------------------------------------------------------------------------
*/

#include "device.h"
#include "ioctls.h"
#include "methods.h"
#include "spaceconfig.h"

OS_STATUS
ADMXRC_GetSpaceConfig(
  DeviceContext* pDevCtx,
  OS_IO_REQUEST* pReq,
  ADMXRC_IOCTLS_GETSPACECONFIG* pIoctl,
  unsigned long inSize,
  unsigned long outSize)
{
  unsigned int                  space;
  os_uint32                     flags;
  MethodResult                  result;
  
  if (pIoctl == NULL || inSize != sizeof(pIoctl->in) || outSize != sizeof(pIoctl->out)) {
    return OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
  }

  space = pIoctl->in.index;

  /* Read the configuration */
  result = pDevCtx->methods.configSpace(pDevCtx, FALSE, space, &flags);
  if (result == MethodResult_Success) {
    pIoctl->out.flags = flags;
    pReq->status.length = sizeof(pIoctl->out);
    return OS_STATUS_SUCCESS;
  } else if (result == MethodResult_InvalidParameter) {
    return OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
  } else if (result == MethodResult_NotSupported) {
    return OS_MAKE_STATUS(ADMXRC2_NOT_SUPPORTED);
  } else {
    return OS_MAKE_STATUS(ADMXRC2_UNKNOWN_ERROR);
  }
}

OS_STATUS
ADMXRC_SetSpaceConfig(
  DeviceContext* pDevCtx,
  OS_IO_REQUEST* pReq,
  ADMXRC_IOCTLS_SETSPACECONFIG* pIoctl,
  unsigned long inSize)
{
  unsigned int space;
  os_uint32 flags;
  MethodResult result;

  if (pIoctl == NULL || inSize != sizeof(pIoctl->in)) {
    return OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
  }

  space = pIoctl->in.index;
  flags = pIoctl->in.flags;

  /* Write the configuration into registers */
  result = pDevCtx->methods.configSpace(pDevCtx, TRUE, space, &flags);
  if (result == MethodResult_Success) {
    return OS_STATUS_SUCCESS;
  } else if (result == MethodResult_InvalidParameter) {
    return OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
  } else if (result == MethodResult_NotSupported) {
    return OS_MAKE_STATUS(ADMXRC2_NOT_SUPPORTED);
  } else {
    return OS_MAKE_STATUS(ADMXRC2_UNKNOWN_ERROR);
  }
}
